<?php
/**
 * Administration pages (ATOM framework)
 *
 * @link http://digitalnature.ro
 *
 * @package ATOM
 * @subpackage Template
 *
 * @todo: update entire theme settings code to use WP's settings API + xml files for option entries
 */

defined('ATOM') or die();

// hooks
add_action('admin_menu', 'atom_add_menu');
add_action('admin_post_atom_update_settings', 'atom_save_options');
add_action('admin_print_styles', 'atom_settings_css');
add_action('admin_print_scripts', 'atom_settings_js');
add_action('admin_notices', 'atom_editor_warning');
add_action('admin_footer', 'atom_settings_footer');

// ajax hooks
add_action('wp_ajax_site_preview', 'atom_site_preview');
add_action('wp_ajax_process_upload', 'atom_process_upload');
add_action('wp_ajax_latest_news', 'atom_latest_news');
add_action('wp_ajax_process_featured', 'atom_process_featured');
add_action('wp_ajax_create_ad', 'atom_create_ad');

// featured columns
add_filter('manage_posts_columns', 'atom_setup_featured_column_title');
add_filter('manage_media_columns', 'atom_setup_featured_column_title');
add_filter('manage_posts_custom_column', 'atom_setup_posts_column_content', 10, 2);
add_filter('manage_media_custom_column', 'atom_setup_media_column_content', 10, 2);

// theme update check
//add_filter('pre_set_site_transient_update_themes', 'atom_check_update');



/**
 * Check for a new theme versions
 *
 * @since 1.2
 */
// Enable update check on every request. For testing only!
//set_site_transient('update_themes', null);
function atom_check_update($checked_data) {
  global $wp_version;

  if (empty($checked_data->checked)) return $checked_data;

  $theme_base = basename(dirname(dirname(__FILE__)));

  $request = array(
    'slug' => $theme_base,
    'version' => $checked_data->checked[$theme_base.'/'.$theme_base.'.php']
  );

  // Start checking for an update
  $send_for_check = array(
    'body' => array(
      'action' => 'theme_update',
      'request' => serialize($request),
      'api-key' => md5(get_bloginfo('url'))

      // @todo: add username/password check for commercial themes
    ),
    'user-agent' => 'WordPress/'.$wp_version.'; '.get_bloginfo('url')
  );

  $raw_response = wp_remote_post(THEME_UPDATE_URL, $send_for_check);

  if (!is_wp_error($raw_response) && ($raw_response['response']['code'] == 200)) $response = unserialize($raw_response['body']);

  // Feed the update data into WP updater
  if (!empty($response)) $checked_data->response[$theme_base] = $response;
  return $checked_data;
}



/**
 * Get the new version update details
 *
 * @since 1.2
 */
function atom_update_available(&$update_details){
  static $themes_update;

  $theme = get_theme(THEME_NAME); // need to test this

  if (!isset($themes_update)) $themes_update = get_site_transient('update_themes');

  if (is_object($theme) && isset($theme->stylesheet)) $stylesheet = $theme->stylesheet;
  elseif (is_array($theme) && isset($theme['Stylesheet'])) $stylesheet = $theme['Stylesheet'];
  else return false;

  if (isset($themes_update->response[$stylesheet])):
    $update = $themes_update->response[$stylesheet];

    $update_details['theme_name'] = is_object($theme) ? $theme->name : (is_array($theme) ? $theme['Name'] : '');
    if(!current_user_can('update_themes') || empty($update->package))
      $update_details['update_url'] = wp_nonce_url('update.php?action=upgrade-theme&amp;theme='.urlencode($stylesheet), 'upgrade-theme_'.$stylesheet);
    $update_details['new_version'] = $update['new_version'];
    $update_details['details_url'] = add_query_arg(array('TB_iframe' => 'true', 'width' => 1024, 'height' => 800), $update['url']);

    return true;
  endif;
  return false;
}



/**
 * Displays a warning message telling the user he shouldn't edit the theme files :)
 *
 * @since 1.0
 */
function atom_editor_warning(){
  global $current_screen;
  if($current_screen->id == 'theme-editor'): ?>
  <div class="error fade">
   <p>
   <?php _ae('Editing theme files is not a good idea because you will use any changes you make here when you update.'); ?>
   </p>
   <p>
   <?php
     printf(_a('Instead, use the %1$s or %2$s fields from the theme settings.'),
               '<a href="'.admin_url('admin.php?page='.ATOM.'_css').'">'._a("User CSS").'</a>',
               '<a href="'.admin_url('admin.php?page='.ATOM.'_advanced').'">'._a("Custom Functions").'</a>');
   ?>
   </p>
  </div>
  <?php
  endif;
}



/**
 * Set up the theme settings menu
 *
 * @since 1.0
 */
function atom_add_menu(){

  // top level menu
  add_menu_page(
    sprintf(_a('%s settings'), THEME_NAME),
    THEME_NAME,
    'edit_theme_options',        // show the theme settings to all users who can switch themes
    ATOM,
    'atom_settings',        // function below
    THEME_URL.'/core/admin/i/atom-icon.png',
    61                      // show it after Appearance
  );

  // sub menus
  add_submenu_page(ATOM, sprintf(_a('%s &raquo; Welcome!'), THEME_NAME), THEME_NAME, 'edit_theme_options', ATOM, 'atom_settings');
  add_submenu_page(ATOM, sprintf(_a('%s &raquo; Design Tweaks'), THEME_NAME), _a('Design'), 'edit_theme_options', ATOM.'_design', 'atom_settings');
  add_submenu_page(ATOM, sprintf(_a('%s &raquo; Content Options'), THEME_NAME), _a('Content Options'), 'edit_theme_options', ATOM.'_content', 'atom_settings');

  if(atom_is_option_enabled('advertisments'))
    add_submenu_page(ATOM, sprintf(_a('%s &raquo; Manage Advertisments'), THEME_NAME), _a('Ad Blocks'), 'edit_theme_options', ATOM.'_ads', 'atom_settings');

  if(atom_is_option_enabled('css'))
    add_submenu_page(ATOM, sprintf(_a('%s &raquo; User CSS code'), THEME_NAME), _a('User CSS'), 'edit_theme_options', ATOM.'_css', 'atom_settings');

  add_submenu_page(ATOM, sprintf(_a('%s &raquo; Advanced Settings'), THEME_NAME), _a('Advanced'), 'edit_theme_options', ATOM.'_advanced', 'atom_settings');

  add_thickbox();

  atom("add_menu");
}



/**
 * Get the current theme settings page.
 * I'm sure there's a better way to do this
 *
 * @since 1.0
 *
 * @return string The page slug
 */
function atom_get_current_settings_page(){
  global $current_screen;

  // not a theme settings page
  if(!strstr($current_screen->id, ATOM)) return false;

  // get the current settings page; wp should really provide a cleaner page title :(
  list($junk, $page) = explode("_page_", $current_screen->id);

  if($page == ATOM) return ATOM;
  else return str_replace(ATOM.'_', '', $page);
}



/**
 * Include admin/interface.css in the admin styles
 *
 * @since 1.0
 */
function atom_settings_css($dir){
  wp_enqueue_style(ATOM.'-settings', THEME_URL.'/core/admin/interface.css');
}



/**
 * Include js
 *
 * @since 1.0
 */
function atom_settings_js(){
  global $current_screen;
  $page = atom_get_current_settings_page();
  // only load if we're one of the theme settings pages -- @todo: need to improve this
  if($page || in_array($current_screen->id, array('widgets', 'edit-post', 'upload'))):
    wp_enqueue_script('jquery');
    wp_enqueue_script('interface', THEME_URL.'/core/admin/interface.js');

    if(in_array($page, array('advanced', 'css', 'content')))
      wp_enqueue_script('codemirror', THEME_URL.'/core/admin/codemirror/js/codemirror.js');

    wp_enqueue_script('post-message', THEME_URL.'/core/admin/pm.js');
  endif;
}



/**
 * Load the site preview iframe with ajax (faster access to theme settings)
 *
 * @since 1.0
 */
function atom_site_preview() {
  check_ajax_referer("theme-settings"); ?>
  <iframe id="themepreview" name="themepreview" src="<?php echo home_url(); ?>/?themepreview=1"></iframe>
  <?php exit();
}



/**
 * Initialization js
 *
 * @since 1.0
 */
function atom_settings_footer(){
  global $current_screen;
  if(atom_get_current_settings_page() || in_array($current_screen->id, array('widgets', 'edit-post', 'upload'))):
  $current_settings = atom_get_options();
  $nonce = wp_create_nonce('theme-settings');
  ?>
  <script type="text/javascript">
  /*<![CDATA[*/
  jQuery(document).ready(function($){

    $(".create-ad").click(function(event){
      event.preventDefault();

      var id = 'a'+(new Date()).getTime();

      $.ajax({
        url: '<?php echo admin_url("admin-ajax.php"); ?>',
        type: "GET",
        context: this,
        data: ({
          action: 'create_ad',
          key: id,
          _ajax_nonce: '<?php echo $nonce; ?>'
        }),
        beforeSend: function() {
          $(this).attr("disabled","disabled");
          $(this).parent().addClass('loading');
        },

        success: function(data){
          $(this).removeAttr("disabled");
          $(this).parent().removeClass('loading');
          var block = $(data);
          block.filter('#ad_'+id).hide();
          $("#ad-blocks").prepend(block);
          $("#ad_"+id).animate({
                    "opacity": "show",
                    "height": "show",
                    "marginTop": "show",
                    "marginBottom": "show",
                    "paddingTop": "show",
                    "paddingBottom": "show"
                   }, 333);

        }

      });

    });

    // style splitter
    $("div[id*='atom-splitter-']").each(function(){
      $(this).addClass("atom-widget-splitter");
      $('.widget-control-actions .alignright', this).remove();
    })

    /*/ expand all widget areas by default
    $('#widgets-right').children('.widgets-holder-wrap').children('.sidebar-name').each(function(){
      var c = $(this).siblings('.widgets-sortables'), p = $(this).parent();
      p.removeClass('closed');
      c.sortable('enable').sortable('refresh');
    });
    //*/

    // simple tabbed interface
    $(".atom-tabs .nav").delegate("a", "click", function(){
      var tabs = $(this).closest(".atom-tabs");

      // get the ID of the element we need to show
      var target = $(this).attr("href").split('#')[1];

      $(".nav a", tabs).removeClass("active");
      $(this).addClass("active");

      $("input.active-tab", tabs).val(target);

      //hide the tabs that doesn't match the ID
      $('.tab:not(.'+target+')', tabs).hide();
      $('.tab.'+target, tabs).show();
     return false;
    });

    // visibility options button (fix wp save bug)
    $("#widgets-right .visibility-options.needfix").livequery(function(){
      var widget = $(this).closest('div.widget');
      wpWidgets.save(widget, 0, 1, 0);
      return false;
    });

    <?php if(current_user_can('edit_posts')): ?>
    $(".featured a").click(function() {
      var pos = $(this).attr('id').lastIndexOf('-');

      $.ajax({
        url: '<?php echo admin_url("admin-ajax.php"); ?>',
        type: "GET",
        context: this,
        data: ({
          action: 'process_featured',
          id: $(this).attr('id').substr(++pos),
          isOn: ($(this).hasClass('on') ? 1 : 0),
          what: $(this).attr('rel'),
          _ajax_nonce: '<?php echo $nonce; ?>'
        }),
        beforeSend: function() {
          $(this).attr("class", "loading");
        },

        error: function(request){
          alert('<?php echo esc_js(_a("Error while featuring posts")); ?>');
          $(this).attr('class', "error");
        },

        success: function(data) {
          $(this).attr('class', data);
        }

      });
    });
    <?php endif; ?>

    // input dependancies (disable/enable them based on the rules attribute check)
    $(".atom-block [followRules]").setupDependencies();

    <?php if(atom_is_option_enabled('layout')): ?>

    // dimensions, common variables
    // @todo: replace slider with a column splitter plugin
    var scale_grid_12 = ['|', '20', '|', '60', '|', '|', '|', '140', '|', '|', '|', '220', '|', '|', '|', '300', '|', '|', '|', '380', '|', '|', '|', '460', '|', '|', '|', '540', '|', '|', '|', '620', '|', '|', '|', '700', '|', '|', '|', '780', '|', '|', '|', '860', '|', '|', '|', '940', '|'],
        scale_fluid = ['0', '|', '10', '|', '20', '|', '30', '|', '40', '|','50', '|', '60', '|', '70', '|', '80', '|', '90', '|', '100'],
        page_width = '<?php echo($current_settings['page_width'] == 'fluid' ? 'fluid' : 'fixed'); ?>',
        layout = '<?php echo $current_settings['layout']; ?>',
        unit = '<?php echo($current_settings['page_width'] == 'fluid' ? '%' : 'px'); ?>',
        gs = <?php echo($current_settings['page_width'] == 'fluid' ? '100' : '960'); ?>,
        jstep = <?php echo($current_settings['page_width'] == 'fluid' ? '1' : '10'); ?>,
        jscale = <?php echo($current_settings['page_width'] == 'fluid' ? 'scale_fluid' : 'scale_grid_12'); ?>,
        set_slider = function(){
          $("#dimensions .current").html('');
          layout = $("input[name='layout']").val();
          if(layout != 'col-1'){
            var current_dimensions = $('input#<?php echo ATOM; ?>_dimensions_'+page_width+'_'+layout).val();
            $("#dimensions .current").html('<input class="slider" type="hidden" value="'+current_dimensions+'" />');
            $("input.slider").slider({
              from: 0,
              to: gs,
              step: jstep,
              dimension: unit,
              scale: jscale,
              limits: false,
              onstatechange: function(value){
                pm({ // live preview
                  target: window.frames["themepreview"],
                  type: 'dimensions',
                  data: {layout: layout, sizes: value, unit: unit, gs: gs}
                });
                $('input#<?php echo ATOM; ?>_dimensions_'+page_width+'_'+layout).val(value);
                return value;
              }
            });
          }else{
            $("#dimensions .current").html('<label><small><?php _ae("Nothing to configure in single column mode"); ?></small></label>');
              pm({ // live preview
                target: window.frames["themepreview"],
                type: 'dimensions',
                data: {layout: layout, sizes: '0', unit: unit, gs: gs}
              });
          }
        };

    // dimensions (column sizes)
    set_slider();

    // page width options
    $("input[name='page_width']").change(function(){
      page_width = $(this).val();
      unit = (page_width == 'fluid') ? '%' : 'px';
      gs = (page_width == 'fluid') ? 100 : 960;
      jstep = (page_width == 'fluid') ? 1 : 10;
      jscale = (page_width == 'fluid') ? scale_fluid : scale_grid_12;
      pm({ // live preview
        target: window.frames["themepreview"],
        type: 'page_width',
        data: page_width
      });
      set_slider();
    });

    // max page width field
    $("input[name='page_width_max']").bind('keyup mouseup change',function(e){
      page_width_max = $(this).val();
      pm({ // live preview
        target: window.frames["themepreview"],
        type: 'page_width_max',
        data: page_width_max
      });
      set_slider();
    });

    <?php endif; ?>

    // allow select type operations on links (nicer selects)
    $('a.select').click(function(){
      var button = $(this);
      var option_type = $(this).parent().attr('id');
      var selected_option = $(this).attr('rel');
      button.parent().find("input.select").val(selected_option);
      button.parent().find('a').removeClass("active");
      button.addClass("active");
      if(option_type == 'layout') set_slider();
      pm({ // live preview
        target: window.frames["themepreview"],
        type: option_type,
        data: selected_option
      });
      return false;
    });

    // hidden input field (used for creating the nicer select inputs above)
    // not to be confused with input[type=select]
    $("input.select").change(function(){
      var data = $(this).val();
      $("a.select[rel='"+data+"']").addClass("active");
    }).change();


    // color picker
    $('.color-selector').ColorPicker({
      onShow: function (colpkr) {
        $(colpkr).css({ opacity: 0, marginTop: -10 }).show().animate({ opacity: 1, marginTop: 0 }, 160);
        $(this).ColorPickerSetColor($(this).find('input').val());
        return false;
      },
      onHide: function (colpkr) {
        $(colpkr).fadeOut(160);
        return false;
      },
      onChange: function (hsb, hex, rgb, el) {
        $(el).find(".preview").css('background-color', '#'+hex);
        $("input", el).val(hex);
        pm({ // live preview
          target: window.frames["themepreview"],
          type: "background-color",
          data: '#'+hex
        });
      }
    });

    // image upload buttons
    $('a.upload').each(function(){
      var button = $(this);
      var option_name = $(this).attr('rel');
      var button_id = $(this).attr('id');
      new AjaxUpload(button_id, {
        action: '<?php echo admin_url("admin-ajax.php"); ?>',
        name: option_name, // file upload name
        data: { // Additional data to send
        action: 'process_upload',
        type: 'upload',
        _ajax_nonce: '<?php echo $nonce; ?>',
        data: option_name },
        autoSubmit: true, // Submit file after selection
        responseType: false,
        onChange: function(file, extension){},
        onSubmit: function(file, extension){
          button.text('<?php _ae("Uploading"); ?>'); // change button text, when user selects file
          this.disable(); // If you want to allow uploading only 1 file at time, you can disable upload button
          interval = window.setInterval(function(){
            var text = button.text();
            if (text.length < 13){ button.text(text + '.'); }
            else { button.text('<?php _ae("Uploading"); ?>'); }
          }, 200);
        },
        onComplete: function(file, response) {
          alert(response);
          var obj = eval('('+response+')'); // the response is a json string. convert it to a array
          window.clearInterval(interval);
          this.enable(); // enable upload button
          if(obj.error != ''){ // we have a error
            $("div.error.upload").remove(); // remove the old error messages, if they exists
            $("#theme-settings-form").prepend('<div class="error upload"><p>'+obj.error+'</p></div>');
            button.text('<?php _ae("Try Another Image?"); ?>');
            $("html").animate({scrollTop:0}, 333);
          } else {
            $("div.error.upload").remove();
            $("img", button.parent()).animate({ opacity: 0, top: -100 }, 200);
            $("img", button.parent()).attr('src', obj.url).animate({ opacity: 1, top: 0 }, 200);

            $("a.reset_upload", button.parent()).fadeIn(150);
            button.text('<?php _ae("Change Image"); ?>');
            $("input[name='"+option_name+"']").val(obj.url);
            pm({ // live preview
              target: window.frames["themepreview"],
              type: option_name,
              data: obj.url
            });
          }
        }
      });
    });

    // remove uploaded image button
    $('a.reset_upload').click(function(){
      var button = $(this);
      var option_name = $(this).attr('rel');
      var button_id = $(this).attr('id');
      var data = {
        action: 'process_upload',
        type: 'reset_upload',
        _ajax_nonce: '<?php echo $nonce; ?>',
        data: option_name
      };
      $.post('<?php echo admin_url("admin-ajax.php"); ?>', data, function(response){
        $('#image-' + option_name).animate({ opacity: 0, top: -100 }, 200, function(){
           $(this).attr('src', '<?php echo THEME_URL; ?>/core/admin/i/x.gif');
        });
        button.fadeOut(150);
        $("a.upload", button.parent()).text('<?php _ae("Upload Image"); ?>');
        $("input[name='"+option_name+"']").val('');
        $("input[name='"+option_name+"']").val('');
        $("div.error.upload").remove();
        pm({ // live preview
          target: window.frames["themepreview"],
          type: option_name,
          data: 'remove'
        });
      });
      return false;
    });

    // latest news rss
    $("#latest-news").each(function(){
      $.ajax({
        type: "post",
        url: "admin-ajax.php",
        data: { action: 'latest_news', _ajax_nonce: '<?php echo $nonce; ?>' },
        context: this,
        success: function(data){
          $(".inside", this).html(data);
          $("ul", this).animate({ opacity: 'show', height: 'show' }, 200).show();
        }
      });
    });

    // preview document
    $("#themepreview-wrap").each(function(){
      $.ajax({
        type: "post",
        url: "admin-ajax.php",
        data: { action: 'site_preview', _ajax_nonce: '<?php echo $nonce; ?>' },
        context: this,
        beforeSend: function() { $(".loading", this).show("slow");},
        complete: function() { $(".loading", this).hide("fast");},
        success: function(html){
          $(this).html(html);
          $(this).show();
          //jQuery('#themepreview-wrap:not(.processed)').TextAreaResizer();
          $('#themepreview').load(function(){ // wait for load because of IE problems...
            frame = $(this);
            var parent_url = decodeURIComponent(document.location.hash.replace(/^#/, '')),link;
            jQuery('#themepreview').contents().find("a").each(function(){
              $href = jQuery(this).attr('href'); // remove links to other pages from iframe doc.
              if ($href && $href.indexOf("#") != 0) jQuery(this).attr("href","#");
            });
          });
        }
      });
    });

  });

  /*]]>*/
  </script>
  <?php
  endif;
}



/**
 * Image upload processing (AJAX)
 * needed for logo, background etc. settings
 *
 * @since 1.0
 *
 * @todo: add ability to the edit the image (wp crop/resize)
 */
function atom_process_upload() {
  check_ajax_referer("theme-settings");
  if(!current_user_can('upload_files')) $error = _a('You are not authorised to perform this operation.');
  $error = '';
  $url = '';
  $option_name = esc_attr($_POST['data']);
  if($_POST['type'] == 'upload'):
     $filename = $_FILES[$option_name];
     $uploaded_file = wp_handle_upload($filename, array("test_form" => false, "action" => 'wp_handle_upload', "mimes" => array('jpg|jpeg|jpe' => 'image/jpeg', 'png' => 'image/png')));
     if(!empty($uploaded_file['error'])):
      $error = sprintf(_a('Upload Error: %s'), $uploaded_file['error']);
     else:

       $imageinfo = getimagesize($uploaded_file['file']);
       list($width, $height) = $imageinfo;

       $url = $uploaded_file['url'];
     endif;
  elseif($_POST['type'] == 'reset_upload'):
     $options[$option_name] = '';
  endif;

  echo json_encode(array('error' => $error, 'url' => $url));
  die();
}



/**
 * Recursively sanitizes theme options (input)
 *
 * @since 1.3
 *
 * @param mixed $option
 *
 * @return mixed
 */
function atom_sanitize_option($option){
  if(is_array($option)):  // could be replaced with array_walk_recursive() in PHP > 5
   foreach($option as $key => $value)
     $option[$key] = atom_sanitize_option($value);
   return $option;
  endif;

  if(!current_user_can('unfiltered_html')) return stripslashes(wp_filter_post_kses((string)$option)); else return stripslashes((string)$option);
}



/**
 * Theme settings update function
 *
 * @since 1.0
 */
function atom_save_options() {
  check_admin_referer('theme-settings');
  $error = false;

  // check permissions
  if(!current_user_can('edit_theme_options')) wp_die(_a('You are not authorised to perform this operation.'));
  $options = atom_get_options();

  // only super admins
  if(!current_user_can('edit_themes') && isset($_POST['functions'])) unset($_POST['functions']);

  // change options
  foreach (atom_default_settings() as $key => $value)
   if(isset($_POST[$key]) && atom_is_option_enabled($key))
     $options[$key] = atom_sanitize_option($_POST[$key]);

  // ads page & no ad blocks? need to improve this...
  if($_POST['current_page'] == ATOM.'_ads' && !isset($_POST['advertisments']))
    $options['advertisments'] = array();

  atom_update_options($options);

  // reset? (super-admins only)
  if(isset($_POST['reset']) && current_user_can('edit_themes')) atom_setup_options();
  
  if($error) wp_redirect(admin_url('admin.php?page='.esc_attr($_POST['current_page']).'&error='.$error));
  else wp_redirect(admin_url('admin.php?page='.esc_attr($_POST['current_page']).'&updated=true'));
}



/**
 * Latest News RSS Feed from digitalnature.ro (AJAX)
 *
 * @since 1.0
 */
function atom_latest_news() {
  check_ajax_referer("theme-settings");
  wp_widget_rss_output('http://digitalnature.ro/feed', array('show_author' => 0, 'show_date' => 1, 'show_summary' => 1, 'items' => 5));
  die();
}



/**
 * Retrieve external theme styles (.css files from the styles folder)
 *
 * @since 1.0
 *
 * @return array|bool A array containing each file meta info, or false if not files found.
 */
function atom_get_schemes(){
  $scheme_headers = array(
    'name' => 'Style Name',
    'color' => 'Color',
    'description' => 'Description',
    'author' => 'Author',
    'version' => 'Version'
  );

  $schemes = array();
  foreach (glob(THEME_STYLES_PATH."/*.css") as $filename)
    if($meta = get_file_data($filename, $scheme_headers)):
     $meta['file'] = basename($filename);
     $schemes[] = $meta;
    endif;

  return (!empty($schemes) ? $schemes : array());
}



function atom_create_ad(){
  check_ajax_referer("theme-settings");
  atom_advertisment_form(strip_tags($_GET['key']));
  die();
}

/**
 * The theme settings pages
 *
 * Note:
 *  Because non-checked input field values are not sent trough $_POST, all options that have checkboxes need to have
 *    <input type="hidden" name="option_name" value="0" />
 *  before the checkbox option. This trick supposes the inputs are always processed sequentially,
 *  so the hidden input value is sent (0) if the checkbox is not checked.
 *
 * @since 1.0
 */
function atom_settings() {
  global $is_IE;

  // only allow users who can at least switch themes to view these pages
  if(!current_user_can('edit_theme_options')) wp_die(_a('You are not authorised to perform this operation.'));

  $errors = array(
              1 => _a("Import failed. Invalid settings. ")
            );

  $parent_theme = get_theme_data(TEMPLATEPATH.'/style.css');
  $update_details = array();
  if(atom_update_available($update_details) && isset($update_details['update_url']))
    $update_link = "(<a href=\"{$update_details['update_url']}\">"._a("Update")."</a>)";

  ?>
  <div id="theme-settings" class="atom-block wrap clear-block">
    <?php screen_icon('themes'); ?><h2><?php echo get_admin_page_title(); ?></h2>
    <form id="theme-settings-form" action="<?php echo admin_url('admin-post.php?action=atom_update_settings'); ?>" method="post" enctype="multipart/form-data">
      <?php

      // new theme version
      if (isset($update_details['new_version'])): ?>
      <div class="updated fade below-h2">
        <p>
         <?php
          if(!isset($update_details['update_url']))
            printf(_a('<strong>There is a new version of %1$s available</strong>. <a href="%2$s" class="thickbox" title="%1$s">View %1$s %3$s details</a> or notify the site administrator.'), $update_details['theme_name'], $update_details['details_url'], $update_details['new_version']);
          else
            printf(_a('<strong>There is a new version of %1$s available</strong>. There is a new version of %1$s available. <a href="%2$s" class="thickbox" title="%1$s">View %1$s %3$s details</a> or <a href="%4$s" >upgrade automatically</a>.'), $update_details['theme_name'], $update_details['details_url'], $update_details['new_version'], $update_details['update_url']);
         ?>
        </p>
      </div>
      <?php endif; ?>

      <?php
      // just updated?
      if (isset($_GET['updated'])): ?>
      <div class="updated fade below-h2">
        <p><?php _ae('Settings saved.'); ?> <a href="<?php echo BLOG_URL; ?>"><?php _ae('View site'); ?></a></p>
      </div>
      <?php endif; ?>

      <?php
      // error?
      if (isset($_GET['error'])): ?>
      <div class="error fade below-h2">
        <p><?php echo $errors[intval($_GET['error'])]; ?></p>
      </div>
      <?php endif; ?>

      <?php
      // intro page
      if(atom_get_current_settings_page() == ATOM): ?>
      <div class="metabox-holder clear-block">
         <div class="alignleft" style="width: 70%">
         <div class="postbox">
           <h3><span><?php _ae("General Information"); ?></span></h3>
           <div class="inside clear-block">

              <ul class="ul-disc">
                <li><?php printf(_a("Core (ATOM) version: %s"), '<strong>'.ATOM_VERSION.'</strong>'); ?> <?php if(isset($update_link)) echo $update_link; ?></li>
                <li><?php printf(_a("Theme Version: %s"), '<strong>'.THEME_VERSION.'</strong>'); ?> <?php if(is_child_theme()) printf(_a("(Child theme of %s)"), $parent_theme['Name']); ?></li>
              </ul>

              <ul class="ul-disc">
                <li><a href="<?php echo THEME_URL; ?>/doc/" target="_blank"><strong><?php _ae("Theme documentation"); ?></strong></a></li>
                <li><a href="<?php echo THEME_URI; ?>" target="_blank"><?php _ae("Project Homepage"); ?></a></li>
              </ul>
           </div>
         </div>

         <?php $langs = get_available_languages(TEMPLATEPATH.'/lang'); ?>
         <?php if(!empty($langs)): ?>
         <div class="postbox">
           <h3><span><?php _ae("Available languages"); ?></span></h3>
           <div class="inside clear-block">

             <ul class="ul-disc">
             <?php
              $mo = new MO();
              foreach($langs as $lang):
                @$mo->import_from_file(TEMPLATEPATH.'/lang/'.$lang.'.mo'); //print_r($mo->headers);
                if(empty($mo->headers)) continue;
                $translator = preg_replace('/(?<!href=")http:\/\//','', make_clickable(strip_tags($mo->headers['Last-Translator'])));
                ?>
                <li>
                  <?php printf(_a('%1$s, translator: %2$s (%3$s)'), strip_tags($mo->headers['X-Poedit-Language']), '<strong>'.$translator.'</strong>', '<em>'.strip_tags($mo->headers['Project-Id-Version']).'</em>'); ?>
                </li>
                <?php
              endforeach;
             ?>

             </ul>
           </div>
         </div>
         <?php endif; ?>

         <?php if(current_user_can('edit_themes')): ?>
         <div class="postbox">
           <h3><span><?php _ae("Import/Export/Reset or Backup Theme Options"); ?></span></h3>
           <div class="inside clear-block">
             <label><small><?php _ae("If you wish to import or export theme settings from another website, you may do so here. Below are the current settings, stored as a encoded string"); ?></small></label>
             <textarea readonly="readonly" spellcheck="false" cols="6" rows="24" class="code" name="import_data"><?php echo chunk_split(base64_encode(serialize(atom_get_options())), 50, "\n"); ?></textarea>
             <p class="alignright">
               <input type="submit" class="button-primary" name="reset" value="<?php _ae("Reset options to defaults"); ?>" onclick="if(confirm('<?php _ae("Reset all theme settings to the default values? Are you sure?"); ?>')) return true; else return false;" />
               <input type="submit" name="import" class="button-primary" value="<?php _ae("Import options"); ?>" onclick="if(confirm('<?php _ae("Import these settings? Are you sure?"); ?>')) return true; else return false;"  />
             </p>
             <p>Note: Import works only in the official release of the theme (from the project page). This is because the WordPress Theme Directory is apparently run by amateurs :(</p>
           </div>
         </div>
         <?php endif; ?>
         </div>

         <div class="alignright" style="width: 30%">

         <div class="postbox hide-if-no-js" id="latest-news">
           <h3><span><?php printf(_a("Latest News from %s"), '<a style="display:inline;" href="http://digitalnature.ro">digitalnature.ro</a>'); ?></span></h3>
           <div class="inside clear-block">
              <div class="loading"></div>
           </div>
         </div>
        </div>

      </div>
      <?php atom("settings_main"); ?>
      <?php
      // design
      elseif(atom_get_current_settings_page() == 'design'): ?>
      <div class="hide-if-no-js">
        <?php if(atom_get_options('jquery')): ?>
        <div id="themepreview-wrap"><div class="clear-block"><div class="loading"></div></div></div>
        <?php endif; ?>
        <div class="metabox-holder flow clear-block">

          <?php if(atom_is_option_enabled('layout')): ?>

          <div class="postbox wide">
             <h3><span><?php _ae("Layout Style (Global)"); ?></span></h3>
             <div class="inside clear-block">
               <div class="layout-selector" id="layout">
                 <a href="#" rel="col-1" class="select col-1" title="<?php _ae("One column (No sidebars)"); ?>"></a>
                 <a href="#" rel="col-2-left" class="select col-2-left" title="<?php _ae("Two columns, left sidebar"); ?>"></a>
                 <a href="#" rel="col-2-right" class="select col-2-right" title="<?php _ae("Two columns, right sidebar"); ?>"></a>
                 <a href="#" rel="col-3" class="select col-3" title="<?php _ae("Three columns"); ?>"></a>
                 <a href="#" rel="col-3-left" class="select col-3-left" title="<?php _ae("Three columns, sidebars to the left"); ?>"></a>
                 <a href="#" rel="col-3-right" class="select col-3-right" title="<?php _ae("Three columns, sidebars to the right"); ?>"></a>
                 <input type="hidden" class="select" name="layout" value="<?php echo atom_get_options("layout"); ?>" />

                 <label class="alignleft"><small><?php printf(_a("Note that you can apply different layout styles to individual pages or posts as well (select the appropriate template when editing pages, or use the %s custom field for posts)"), '<code>layout</code>'); ?></small></label>

               </div>

             </div>
          </div>

          <div class="postbox wide">
             <h3><span><?php _ae("Column Sizes"); ?></span></h3>
             <div class="inside clear-block" id="dimensions">
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fixed_col-2-left" name="dimensions_fixed_col-2-left" value="<?php echo atom_get_options('dimensions_fixed_col-2-left'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fixed_col-2-right" name="dimensions_fixed_col-2-right" value="<?php echo atom_get_options('dimensions_fixed_col-2-right'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fixed_col-3" name="dimensions_fixed_col-3" value="<?php echo atom_get_options('dimensions_fixed_col-3'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fixed_col-3-left" name="dimensions_fixed_col-3-left" value="<?php echo atom_get_options('dimensions_fixed_col-3-left'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fixed_col-3-right" name="dimensions_fixed_col-3-right" value="<?php echo atom_get_options('dimensions_fixed_col-3-right'); ?>" />

                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fluid_col-2-left" name="dimensions_fluid_col-2-left" value="<?php echo atom_get_options('dimensions_fluid_col-2-left'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fluid_col-2-right" name="dimensions_fluid_col-2-right" value="<?php echo atom_get_options('dimensions_fluid_col-2-right'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fluid_col-3" name="dimensions_fluid_col-3" value="<?php echo atom_get_options('dimensions_fluid_col-3'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fluid_col-3-left" name="dimensions_fluid_col-3-left" value="<?php echo atom_get_options('dimensions_fluid_col-3-left'); ?>" />
                <input type="hidden" id="<?php echo ATOM; ?>_dimensions_fluid_col-3-right" name="dimensions_fluid_col-3-right" value="<?php echo atom_get_options('dimensions_fluid_col-3-right'); ?>" />

                <div class="current"><?php // the slider input is added with jquery ?></div>
             </div>
          </div>

          <?php endif; ?>

          <div class="block alignleft">

            <?php if(atom_is_option_enabled('layout')): ?>

            <div class="postbox">
               <h3><span><?php _ae("Page Width"); ?></span></h3>
               <div class="inside clear-block">

                 <div class="entry">
                   <label for="<?php echo ATOM; ?>_page_width_fixed">
                     <input name="page_width" id="<?php echo ATOM; ?>_page_width_fixed" type="radio" followRules class="radio" value="fixed" <?php checked('fixed', atom_get_options('page_width')); ?> />
                     <?php printf(_a("Fixed (%s)"), '<a href="http://960.gs">960gs</a>'); ?>
                   </label>
                 </div>

                 <div class="entry">
                   <label for="<?php echo ATOM; ?>_page_width_fluid">
                     <input name="page_width" id="<?php echo ATOM; ?>_page_width_fluid" type="radio" followRules class="radio" value="fluid" <?php checked('fluid', atom_get_options('page_width')); ?> />
                     <?php printf(_a("Fluid, but not more than %s pixels"), '<label><input size="4" followRules rules="DEPENDS ON page_width BEING fluid" name="page_width_max" id="'.ATOM.'_page_width_max" type="text" value="'.atom_get_options('page_width_max').'" /></label>'); ?>
                   </label>
                 </div>

                 <div class="entry">
                   <label class="disabled" title="Feature not yet implemented">
                     <input disabled="disabled" type="radio" class="radio disabled" />
                     <?php _ae("Semi-Fluid (Fixed Sidebars)"); ?>
                   </label>
                 </div>

               </div>
            </div>

            <?php endif; ?>

            <?php if(atom_is_option_enabled('logo')): ?>
            <div class="postbox upload-block">
               <h3><span><?php _ae("Site Title (Logo)"); ?></span></h3>
               <div class="inside clear-block">

                <img id="image-logo" class="image-upload" src="<?php echo ($logo = atom_get_options('logo')) ? $logo : THEME_URL.'/core/admin/i/x.gif'; ?>" title="<?php _ae("Current logo image"); ?>" />
                <a class="button reset_upload alignright <?php if(!$logo): ?>hidden<?php endif; ?>" id="<?php echo ATOM; ?>_reset_logo" rel="logo"><?php _ae("Remove"); ?></a>
                <a class="button upload alignright" id="<?php echo ATOM; ?>_logo" rel="logo"><?php _ae($logo ? ("Change Image") : ("Upload Image")); ?></a>
                <input type="hidden" name="logo" value="<?php echo $logo; ?>" />
               </div>
            </div>
            <?php endif; ?>
          </div>

          <div class="block alignleft">

            <?php if(atom_is_option_enabled('color_scheme')): ?>
            <div class="postbox">
               <h3><span><?php _ae("Styles &amp; colors"); ?></span></h3>
               <div class="inside clear-block">

                 <div class="clear-block colorscheme-selector" id="color-scheme">
                   <?php
                    foreach (atom_get_schemes() as $scheme):
                     echo '<a href="#" rel="'.$scheme['file'].'" class="select" style="background-color:'.$scheme['color'].'" title="'.sprintf(_a('%1$s by %2$s'), $scheme['name'], $scheme['author']).'">'.$scheme['name'].'</a>';

                    endforeach;
                   ?>
                   <a href="#" rel="" class="select default" title="<?php _ae("Blank"); ?>"><?php _ae("Disabled, use Custom or Child theme styles"); ?></a>
                   <input type="hidden" class="select" name="color_scheme" value="<?php echo atom_get_options("color_scheme"); ?>" />
                 </div>
               </div>
            </div>
            <?php endif; ?>

            <?php if(atom_is_option_enabled('background_image') || atom_is_option_enabled('background_color')): ?>
            <div class="postbox upload-block">
               <h3><span><?php _ae("Page Background"); ?></span></h3>
               <div class="inside clear-block">

               <?php if(atom_is_option_enabled('background_image')): ?>
                <label><small><?php _ae("Note that some theme styles might override this setting"); ?></small></label>
                <img id="image-background_image" class="image-upload" src="<?php echo ($background = atom_get_options('background_image')) ? $background : THEME_URL.'/core/admin/i/x.gif'; ?>" title="<?php _ae("Current background image"); ?>" />

                 <a class="button reset_upload alignright <?php if(!$background): ?>hidden<?php endif; ?>" id="<?php echo ATOM; ?>_reset_background" rel="background_image"><?php _ae("Remove"); ?></a>
                 <a class="button upload alignright" id="<?php echo ATOM; ?>_background" rel="background_image"><?php _ae($background ? ("Change Image") : ("Upload Image")); ?></a>
                 <input type="hidden" name="background_image" value="<?php echo $background; ?>" />
               <?php endif; ?>

               <?php if(atom_is_option_enabled('background_color')): ?>
                 <div class="color-selector alignleft" id="opt_background_color">
                   <div class="preview" style="background-color: #<?php echo esc_html(atom_get_options('background_color')); ?>">
                     <input name="background_color" type="hidden" value="<?php echo esc_html(atom_get_options('background_color')); ?>" />
                   </div>
                 </div>
               <?php endif; ?>

               </div>
            </div>
            <?php endif; ?>

            <?php if(atom_is_option_enabled('favicon')): ?>
            <div class="postbox upload-block">
               <h3><span><?php _ae("Favicon"); ?></span></h3>
               <div class="inside clear-block">

                <img id="image-favicon" class="alignleft image-upload" src="<?php echo ($favicon = atom_get_options('favicon')) ? $favicon : THEME_URL.'/core/admin/i/x.gif'; ?>" title="<?php _ae("Current favicon"); ?>" width="16" height="16" />

                <a class="button reset_upload alignright <?php if(!$favicon): ?>hidden<?php endif; ?>" id="<?php echo ATOM; ?>_reset_favicon" rel="favicon"><?php _ae("Remove"); ?></a>
                <a class="button upload alignright" id="<?php echo ATOM; ?>_favicon" rel="favicon"><?php _ae($favicon ? ("Change Image") : ("Upload Image")); ?></a>
                <input type="hidden" name="favicon" value="<?php echo $favicon; ?>" />
               </div>
            </div>
            <?php endif; ?>

          </div>

        </div>
      </div>
      <div class="hide-if-js">
        <?php _ae("Sorry, this page requires JavaScript to be enabled in your browser :("); ?>
      </div>
      <?php atom("settings_design"); ?>

      <?php
      // content settings page
      elseif(atom_get_current_settings_page() == 'content'): ?>
      <div class="metabox-holder flow clear-block">

         <?php if(atom_is_option_enabled('footer_content')): ?>

         <div class="postbox wide clear-block">
           <h3><span><?php _ae("Footer content"); ?></span></h3>
           <div class="inside">
            <p>
           <textarea rows="10" cols="60" name="footer_content" id="<?php echo ATOM; ?>_footer_content" class="code widefat"><?php echo esc_html(atom_get_options('footer_content')); ?></textarea>
           <script type="text/javascript">
            var editor = CodeMirror.fromTextArea('<?php echo ATOM; ?>_footer_content', {
              height: "202px",
              parserfile: ["parsexml.js", "parsecss.js", "tokenizejavascript.js", "parsejavascript.js", "parsehtmlmixed.js"],
              stylesheet: ["<?php echo THEME_URL; ?>/core/admin/codemirror/css/xmlcolors.css",
                           "<?php echo THEME_URL; ?>/core/admin/codemirror/css/jscolors.css",
                           "<?php echo THEME_URL; ?>/core/admin/codemirror/css/csscolors.css"],
              path: "<?php echo THEME_URL; ?>/core/admin/codemirror/js/",
              lineNumbers: true,
              textWrapping: false
            });
           </script>
           </p>
           <label>
             <small>
              <?php  if(current_user_can('unfiltered_html')) _ae("You can add HTML code."); else _ae("HTML is allowed, but some tags may be filtered."); ?>

              <?php printf(_a("Use %s for convenient adjustments"), '<a href="'.THEME_URL.'/doc/" target="_blank">'._a("[shortcodes]").'</a>'); ?>
             </small>
           </label>
           </div>
         </div>
         <?php endif; ?>

         <div class="block alignleft">

           <?php if (atom_is_option_enabled('post')): ?>
           <div class="postbox">
             <h3><span><?php _ae("Post Previews"); ?></span></h3>
             <div class="inside">

               <p><strong><?php _ae("Display"); ?></strong></p>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_title">
                   <input type="hidden" name="post_title" value="0">
                   <input id="<?php echo ATOM; ?>_post_title" name="post_title" type="checkbox" value="1" <?php checked('1', atom_get_options('post_title')) ?> />
                   <?php _ae("Title"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_content">
                   <input type="hidden" name="post_content" value="0">
                   <input id="<?php echo ATOM; ?>_post_content" name="post_content" followRules type="checkbox" value="1" <?php checked('1', atom_get_options('post_content')) ?> />
                   <?php _ae("Content"); ?>
                 </label>
                 <select name="post_content_mode" id="<?php echo ATOM; ?>_post_content_mode" class="followRules" followRules rules="DEPENDS ON post_content">
                    <option value="f" <?php selected('f', atom_get_options('post_content_mode')) ?>><?php _ae("Full post"); ?></option>
                    <option value="ff" <?php selected('ff', atom_get_options('post_content_mode')) ?>><?php _ae("Full post, filtered"); ?></option>
                    <option value="e" <?php selected('e', atom_get_options('post_content_mode')) ?>><?php _ae("Excerpt"); ?></option>
                    <option value="50" <?php selected('50', atom_get_options('post_content_mode')) ?>><?php printf(_a("First %s words, filtered"), 50); ?></option>
                    <option value="100" <?php selected('100', atom_get_options('post_content_mode')) ?>><?php printf(_a("First %s words, filtered"), 100); ?></option>
                    <option value="150" <?php selected('150', atom_get_options('post_content_mode')) ?>><?php printf(_a("First %s words, filtered"), 150); ?></option>
                 </select>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_category">
                   <input type="hidden" name="post_category" value="0">
                   <input id="<?php echo ATOM; ?>_post_category" name="post_category" type="checkbox" value="1" <?php checked('1', atom_get_options('post_category')) ?> />
                   <?php _ae("Category"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_date">
                   <input type="hidden" name="post_date" value="0">
                   <input id="<?php echo ATOM; ?>_post_date" name="post_date" followRules type="checkbox" value="1" <?php checked('1', atom_get_options('post_date')) ?> />
                   <?php _ae("Date/Time"); ?>
                 </label>
                 <select name="post_date_mode" id="<?php echo ATOM; ?>_post_date_mode" followRules rules="DEPENDS ON post_date">
                    <option value="relative" <?php selected('relative', atom_get_options('post_date_mode')) ?>><?php _ae("Relative (Time Since)"); ?></option>
                    <option value="absolute" <?php selected('absolute', atom_get_options('post_date_mode')) ?>><?php _ae("Absolute (Use Time/Date settings)"); ?></option>
                 </select>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_author">
                   <input type="hidden" name="post_author" value="0">
                   <input id="<?php echo ATOM; ?>_post_author" name="post_author" type="checkbox" value="1" <?php checked('1', atom_get_options('post_author')) ?> />
                   <?php _ae("Author"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_comments">
                   <input type="hidden" name="post_comments" value="0">
                   <input id="<?php echo ATOM; ?>_post_comments" name="post_comments" type="checkbox" value="1" <?php checked('1', atom_get_options('post_comments')) ?> />
                   <?php _ae("Comments Link"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_tags">
                   <input type="hidden" name="post_tags" value="0">
                   <input id="<?php echo ATOM; ?>_post_tags" name="post_tags" type="checkbox" value="1" <?php checked('1', atom_get_options('post_tags')) ?> />
                   <?php _ae("Tags"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_thumbs">
                   <input type="hidden" name="post_thumbs" value="0">
                   <input id="<?php echo ATOM; ?>_post_thumbs" name="post_thumbs" followRules type="checkbox" value="1" <?php checked('1', atom_get_options('post_thumbs')) ?> />
                   <?php _ae("Thumbnails"); ?>
                 </label>
                 <select name="post_thumbs_mode" id="<?php echo ATOM; ?>_post_thumbs_mode" followRules rules="DEPENDS ON post_thumbs">
                   <option value="left" <?php selected('left', atom_get_options('post_thumbs_mode')) ?>><?php _ae("Left Aligned"); ?></option>
                   <option value="right" <?php selected('right', atom_get_options('post_thumbs_mode')) ?>><?php _ae("Right Aligned"); ?></option>
                 </select>
               </div>

               <hr />

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_thumb_size">
                   <?php _ae("Thumbnail Size"); ?>
                   <select name="post_thumb_size" id="<?php echo ATOM; ?>_post_thumb_size" followRules rules="DEPENDS ON post_thumbs">
                      <option value="60x60" <?php selected('60x60', atom_get_options('post_thumb_size')) ?>><?php printf(_a("Very Small, %s"), '60 x 60'); ?></option>
                      <option value="90x90" <?php selected('90x90', atom_get_options('post_thumb_size')) ?>><?php printf(_a("Small, %s"), '90 x 90'); ?></option>
                      <option value="120x120" <?php selected('120x120', atom_get_options('post_thumb_size')) ?>><?php printf(_a("Medium, %s"), '120 x 120'); ?></option>
                      <option value="140x140" <?php selected('140x140', atom_get_options('post_thumb_size')) ?>><?php printf(_a("Large, %s"), '140 x 140'); ?></option>
                      <option value="180x180" <?php selected('180x180', atom_get_options('post_thumb_size')) ?>><?php printf(_a("Larger, %s"), '180 x 180'); ?></option>
                      <option value="200x200" <?php selected('200x200', atom_get_options('post_thumb_size')) ?>><?php printf(_a("Very Large, %s"), '200 x 200'); ?></option>
                      <option disabled="disabled">-------------------------------</option>
                      <option value="media" <?php selected('media', atom_get_options('post_thumb_size')); ?>><?php printf(_a("Media setting, %s"), atom_get_options('thumbnail_size_w').' x '.atom_get_options('thumbnail_size_h')); ?></option>

                   </select>
                   <small><?php _ae("Note that the new sized thumbnails require that all images to be resized. See the Advanced page."); ?>
                   </small>

                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_post_thumb_auto">
                   <input type="hidden" name="post_thumb_auto" value="0">
                   <input id="<?php echo ATOM; ?>_post_thumb_auto" name="post_thumb_auto" class="followRules" followRules rules="DEPENDS ON post_thumbs" type="checkbox" value="1" <?php checked('1', atom_get_options('post_thumb_auto')) ?> />
                   <?php _ae("Use first attachment as thumbnail, if none is set"); ?>
                 </label>
               </div>

             </div>
           </div>
           <?php endif; ?>

          <?php if(atom_is_option_enabled('comment')): ?>
           <div class="postbox">
             <h3><span><?php _ae("Comments"); ?></span></h3>
             <div class="inside">

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_comment_flags">
                   <input type="hidden" name="comment_flags" value="0">
                   <input id="<?php echo ATOM; ?>_comment_flags" name="comment_flags" type="checkbox" value="1" <?php checked('1', atom_get_options('comment_flags')) ?> />
                   <?php _ae("Display Country Flags"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_comment_filter">
                   <input type="hidden" name="comment_filter" value="0">
                   <input id="<?php echo ATOM; ?>_comment_filter" name="comment_filter" type="checkbox" value="1" <?php checked('1', atom_get_options('comment_filter')) ?> />
                   <?php _ae("Enable Comment Search"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_comment_karma">
                   <input type="hidden" name="comment_karma" value="0">
                   <input id="<?php echo ATOM; ?>_comment_karma" name="comment_karma" followRules type="checkbox" value="1" <?php checked('1', atom_get_options('comment_karma')) ?> />
                   <?php _ae("Enable Comment Rating (Karma)"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_comment_bury_limit">
                   <?php printf(_a("Hide comments with %s karma and below"), '<label><input size="4" followRules rules="DEPENDS ON comment_karma" name="comment_bury_limit" id="'.ATOM.'_comment_bury_limit" type="text" value="'.atom_get_options('comment_bury_limit').'" /></label>'); ?>
                 </label>
               </div>

             </div>
           </div>
           <?php endif; ?>

         </div>

         <div class="block alignleft">

           <?php if(atom_is_option_enabled('single')): ?>
           <div class="postbox">
             <h3><span><?php _ae("Single Post Page"); ?></span></h3>
             <div class="inside">

               <p><strong><?php _ae("Display"); ?></strong></p>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_single_links">
                   <input type="hidden" name="single_links" value="0">
                   <input id="<?php echo ATOM; ?>_single_links" name="single_links" type="checkbox" value="1" <?php checked('1', atom_get_options('single_links')) ?> />
                   <?php _ae("Previous/Next Links"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_single_print">
                   <input type="hidden" name="single_print" value="0">
                   <input id="<?php echo ATOM; ?>_single_print" name="single_print" type="checkbox" value="1" <?php checked('1', atom_get_options('single_print')) ?> />
                   <?php _ae("Print Link"); ?>
                 </label>
               </div>

               <hr />

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_single_share">
                   <input type="hidden" name="single_share" value="0">
                   <input id="<?php echo ATOM; ?>_single_share" name="single_share" type="checkbox" value="1" <?php checked('1', atom_get_options('single_share')) ?> />
                   <?php _ae("Share Links"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_single_meta">
                   <input type="hidden" name="single_meta" value="0">
                   <input id="<?php echo ATOM; ?>_single_meta" name="single_meta" type="checkbox" value="1" <?php checked('1', atom_get_options('single_meta')) ?> />
                   <?php _ae("Meta Information"); ?>
                 </label>
               </div>

               <hr />

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_single_author">
                   <input type="hidden" name="single_author" value="0">
                   <input id="<?php echo ATOM; ?>_single_author" name="single_author" type="checkbox" value="1" <?php checked('1', atom_get_options('single_author')) ?> />
                   <?php _ae("About the Author"); ?>
                 </label>
               </div>

               <div class="entry">
                 <label for="<?php echo ATOM; ?>_single_related">
                   <input type="hidden" name="single_related" value="0">
                   <input id="<?php echo ATOM; ?>_single_related" name="single_related" type="checkbox" value="1" <?php checked('1', atom_get_options('single_related')) ?> />
                   <?php _ae("Related Posts"); ?>
                 </label>
               </div>

             </div>
           </div>
           <?php endif; ?>

           <?php if (atom_is_option_enabled('media')): ?>
           <div class="postbox">
             <h3><span><?php _ae("Social Media Links"); ?></span></h3>
             <div class="inside">

               <p><em><?php _ae("(Leave blank to disable)"); ?></em></p>

               <?php if (atom_is_option_enabled('media_rss')): ?>
               <div class="entry">
                <p><label for="<?php echo ATOM; ?>_media_rss"><?php _ae("RSS Feed"); ?></label></p>
                <input id="<?php echo ATOM; ?>_media_rss" size="40" name="media_rss" type="text" value="<?php echo atom_get_options('media_rss'); ?>" />
               </div>
               <?php endif; ?>

               <?php if (atom_is_option_enabled('media_twitter')): ?>
               <div class="entry">
                <p><label for="<?php echo ATOM; ?>_media_twitter">Twitter</label></p>
                <input id="<?php echo ATOM; ?>_media_twitter" size="40" name="media_twitter" type="text" value="<?php echo atom_get_options('media_twitter'); ?>" />
               </div>
               <?php endif; ?>

               <?php if (atom_is_option_enabled('media_facebook')): ?>
               <div class="entry">
                <p><label for="<?php echo ATOM; ?>_media_facebook">Facebook</label></p>
                <input id="<?php echo ATOM; ?>_media_facebook" size="40" name="media_facebook" type="text" value="<?php echo atom_get_options('media_facebook'); ?>" />
               </div>
               <?php endif; ?>

               <?php if (atom_is_option_enabled('media_flickr')): ?>
               <div class="entry">
                <p><label for="<?php echo ATOM; ?>_media_flickr">Flickr</label></p>
                <input id="<?php echo ATOM; ?>_media_flickr" size="40" name="media_flickr" type="text" value="<?php echo atom_get_options('media_flickr'); ?>" />
               </div>
               <?php endif; ?>

               <?php if (atom_is_option_enabled('media_myspace')): ?>
               <div class="entry">
                <p><label for="<?php echo ATOM; ?>_media_myspace">MySpace</label></p>
                <input id="<?php echo ATOM; ?>_media_myspace" size="40" name="media_myspace" type="text" value="<?php echo atom_get_options('media_myspace'); ?>" />
               </div>
               <?php endif; ?>

              <?php atom("settings_content_media"); // allow extra options here ?>

             </div>
           </div>
           <?php endif; ?>

         </div>
      </div>

      <?php atom("settings_content"); ?>

      <?php
      // ad settings page
      elseif(atom_get_current_settings_page() == 'ads'): ?>
      <div class="metabox-holder flow ads clear-block">
        <div class="hide-if-no-js">
          <p><input class="button-secondary create-ad" type="submit" value="<?php _ae("Create a new Ad"); ?>" /></p>
          <div id="ad-blocks">
            <?php foreach($ads = (array)atom_get_options('advertisments') as $key => $ad) atom_advertisment_form($key, $ad); ?>
          </div>
        </div>
        <div class="hide-if-js">
          <?php _ae("Sorry, this page requires JavaScript to be enabled in your browser :("); ?>
        </div>

      </div>

      <?php atom("settings_ads"); ?>

      <?php
      // advanced settings page
      elseif(atom_get_current_settings_page() == 'advanced'): ?>

      <?php if(current_user_can('edit_themes') && atom_is_option_enabled('functions')): // only show this if the user can use the theme editor (same type of operation) ?>
      <div class="metabox-holder flow clear-block">


             <label for="<?php echo ATOM; ?>_functions">
              <small>
                <?php _ae("Use this option field if you have plugins that require you to add small pieces of code in the theme's functions.php file."); ?>
                <br />
                <?php _ae("If you need to make extensive customizations you should create your own functions.php file inside a child theme."); ?>
                <br />
                <?php printf(_a("Read the %s for more information."), '<a href="'.THEME_URL.'/doc/" target="_blank">'._a("documentation").'</a>'); ?>
              </small>
             </label>

             <p>
             <textarea rows="26" cols="60" name="functions" id="<?php echo ATOM; ?>_functions" class="code widefat"><?php echo esc_html(atom_get_options('functions')); ?></textarea>
             <script type="text/javascript">
              var editor = CodeMirror.fromTextArea('<?php echo ATOM; ?>_functions', {
                height: "472px",
                parserfile: ["parsexml.js", "parsecss.js", "tokenizejavascript.js", "parsejavascript.js",
                             "../contrib/php/js/tokenizephp.js", "../contrib/php/js/parsephp.js", "../contrib/php/js/parsephphtmlmixed.js"],
                stylesheet: ["<?php echo THEME_URL; ?>/core/admin/codemirror/css/xmlcolors.css",
                             "<?php echo THEME_URL; ?>/core/admin/codemirror/css/jscolors.css",
                             "<?php echo THEME_URL; ?>/core/admin/codemirror/css/csscolors.css",
                             "<?php echo THEME_URL; ?>/core/admin/codemirror/contrib/php/css/phpcolors.css"],
                path: "<?php echo THEME_URL; ?>/core/admin/codemirror/js/",
                lineNumbers: true,
                textWrapping: false
              });
             </script>
             </p>

      </div>
      <?php endif; ?>


        <div class="row clear-block">
           <label for="<?php echo ATOM; ?>_jquery">
            <input type="hidden" name="jquery" value="0">
            <input id="<?php echo ATOM; ?>_jquery" name="jquery" class="checkbox" followRules type="checkbox" value="1" <?php checked('1', atom_get_options('jquery')) ?> />
            <?php _ae("Use jQuery"); ?>
            <small>
              <?php _ae("Only uncheck if you know what you're doing. Your site will run faster without jQuery, but many features won't be available anymore"); ?>
            </small>
           </label>
        </div>
        <div class="row clear-block">
           <label for="<?php echo ATOM; ?>_jquery_init_query">
            <input type="hidden" name="jquery_init_query" value="0">
            <input id="<?php echo ATOM; ?>_jquery_init_query" name="jquery_init_query" followRules rules="DEPENDS ON jquery" class="checkbox" type="checkbox" value="1" <?php checked('1', atom_get_options('jquery_init_query')) ?> />
            <?php _ae("Separate dynamic JavaScript from the HTML document"); ?>
            <small>
              <?php _ae("Generates JS by making a extra Wordpress call (for testing, might be slower than including the JS at the end of the page)"); ?>
            </small>
           </label>
        </div>
        <div class="row clear-block">
           <label for="<?php echo ATOM; ?>_optimize">
            <input type="hidden" name="optimize" value="0">
            <input id="<?php echo ATOM; ?>_optimize" name="optimize" class="checkbox" type="checkbox" value="1" <?php checked('1', atom_get_options('optimize')) ?> />
            <?php _ae("Optimize website for faster loading"); ?>
            <small>
              <?php printf(_a("Performs a series of operations: Loads js in pararell with %s, compresses and concatenates stylesheets"), '<a href="http://headjs.com/">head.js</a>'); ?>
            </small>
           </label>
        </div>

        <div class="row clear-block">
           <?php if(current_user_can('edit_themes')): // just in case the site is running on wp + multiuser + cheap shared host, like wp.digitalnature.eu :) ?>
           <label for="<?php echo ATOM; ?>_generate_thumbs">
           <input type="hidden" name="generate_thumbs" value="0">
           <input id="<?php echo ATOM; ?>_generate_thumbs" name="generate_thumbs" followRules rules="DEPENDS ON jquery" class="checkbox" type="checkbox" value="1" <?php checked('1', atom_get_options('generate_thumbs')) ?> />
           <?php else: ?>
           <input id="<?php echo ATOM; ?>_generate_thumbs" disabled="disabled" class="checkbox disabled" type="checkbox" <?php checked('1', atom_get_options('generate_thumbs')) ?> />
           <label class="disabled" for="<?php echo ATOM; ?>_generate_thumbs" title="<?php _ae("Only Super Admins can change this option"); ?>">
           <?php endif; ?>
            <?php _ae("Automatically create missing thumbnail sizes"); ?>
            <small>
              <?php printf(_a("Asynchronously update thumbnails if needed. You can also use the %s plugin to process all missing thumbnail sizes manually, in a single pass"), '<a href="http://wordpress.org/extend/plugins/regenerate-thumbnails/" target="_blank">Regenerate Thumbnails</a>'); ?>

            </small>
           </label>
        </div>
        <div class="row clear-block">
           <label for="<?php echo ATOM; ?>_lightbox">
            <input type="hidden" name="lightbox" value="0">
            <input id="<?php echo ATOM; ?>_lightbox" name="lightbox" followRules rules="DEPENDS ON jquery" class="checkbox" type="checkbox" value="1" <?php checked('1', atom_get_options('lightbox')) ?> />
            <?php printf(_a("Use theme built-in lightbox (%s) on all image links"), '<a href="http://fancybox.net" target="_blank">Fancybox</a>'); ?>
            <small>
              <?php _ae("Uncheck if you prefer a lightbox plugin"); ?>
            </small>
           </label>
        </div>
        <div class="row clear-block">
           <label for="<?php echo ATOM; ?>_meta_description">
            <input type="hidden" name="meta_description" value="0">
            <input id="<?php echo ATOM; ?>_meta_description" name="meta_description" class="checkbox" type="checkbox" value="1" <?php checked('1', atom_get_options('meta_description')) ?> />
            <?php _ae("Automatically generate meta descriptions"); ?>
            <small>
              <?php _ae("Uncheck if you're using a SEO plugin, otherwise you'll get duplicate fields!"); ?>
            </small>
           </label>
        </div>

        <div class="row clear-block">
           <label for="<?php echo ATOM; ?>_debug">
            <input type="hidden" name="debug" value="0">
            <input id="<?php echo ATOM; ?>_debug" name="debug" class="checkbox" type="checkbox" value="1" <?php checked('1', atom_get_options('debug')) ?> />
            <?php _ae("Display debug messages (english only)"); ?>
            <small>
              <?php _ae("Status notifications about the theme setup (only visible to administrators)"); ?>
            </small>
           </label>
        </div>

        <div class="row clear-block">
           <label for="<?php echo ATOM; ?>_remove_settings">
            <input type="hidden" name="remove_settings" value="0">
            <input id="<?php echo ATOM; ?>_remove_settings" name="remove_settings" class="checkbox" type="checkbox" value="1" <?php checked('1', atom_get_options('remove_settings')) ?> />
            <?php _ae("Theme auto-uninstall"); ?>
            <small>
              <?php printf(_a("Check to remove all %s settings from the database after you switch the theme"), THEME_NAME); ?>
            </small>
           </label>
        </div>


      <?php atom("settings_advanced"); ?>

      <?php
      // user css page
      elseif(atom_get_current_settings_page() == 'css'): ?>
      <div class="metabox-holder flow clear-block">

            <label for="<?php echo ATOM; ?>_css">
             <small>
               <?php printf(_a("Check %s to see existing theme classes and properties, which you can redefine here."),'<a target="_blank" href="'.get_bloginfo('stylesheet_url').'">style.css</a>'); ?>
               <br />
               <?php printf(_a('All URLs must have absolute paths. Use the %1$s keyword to get the parent theme URL (%2$s).'), '<code>{THEME_URL}</code>', THEME_URL); ?>
               <br />
               <?php printf(_a("Read the %s to see all available keywords."), '<a href="'.THEME_URL.'/doc/" target="_blank">'._a("documentation").'</a>'); ?>

               <?php if(is_child_theme()): ?>
               <br />
               <?php printf(_a('To get the child theme URL use %s.'), '<code>{CHILD_THEME_URL}</code>'); ?>
               <?php endif; ?>

             </small>
            </label>
            <p>
            <textarea rows="38" cols="60" name="css" id="<?php echo ATOM; ?>_css" class="code widefat"><?php echo esc_html(atom_get_options('css')); ?></textarea>
            <script type="text/javascript">
             var editor = CodeMirror.fromTextArea('<?php echo ATOM; ?>_css', {
               height: "698px",
               parserfile: ["parsecss.js"],
               stylesheet: ["<?php echo THEME_URL; ?>/core/admin/codemirror/css/csscolors.css"],
               path: "<?php echo THEME_URL; ?>/core/admin/codemirror/js/",
               lineNumbers: true,
               textWrapping: false
             });
            </script>
            </p>
      </div>
      <?php atom("settings_css"); ?>

      <?php endif; ?>

      <div class="save clear-block">
        <p class="controls alignleft">
          <?php if(atom_get_current_settings_page() != ATOM): ?>
          <input type="submit" class="button-primary" name="submit" value="<?php _ae("Save Changes"); ?>" />
          <?php endif; ?>
          <input type="hidden" name="current_page" value="<?php echo ((atom_get_current_settings_page() == ATOM) ? ATOM : ATOM.'_'.atom_get_current_settings_page()); ?>" />
        </p>
      </div>
      <?php wp_nonce_field('theme-settings'); ?>

    </form>

  </div>
  <?php
}



/**
 * Outputs the advertisment block form
 *
 * @since 1.3
 *
 * @param int $id Block ID
 * @param string $ad Ad attributes (serialized string)
 */
function atom_advertisment_form($id, $ad = array(
   'atts' => "page=home&place=after_post&n=1&to=1",
   'html' => "<div class=\"ad-block aligncenter\">\n  <a href=\"http://google.com\"><img src=\"http://dummyimage.com/468x60/000/fff\" alt=\"Test Ad\" /></a>\n</div>")){

  $data = array();
  parse_str($ad['atts'], &$data);

  $data['n'] = min(max(intval($data['n']), 1), atom_get_options('posts_per_page'));

  $ad_context = atom_ad_context();
  $wp_roles = new WP_Roles();
  ?>

 <div class="ad-block clear-block" id="ad_<?php echo $id; ?>">
  <p class="alignleft">
    <label for="ad_<?php echo $id; ?>_page"><?php _ae("Show on:"); ?></label>
    <select followAdRules rel="page" id="ad_<?php echo $id; ?>_page">
      <?php foreach($ad_context['pages'] as $page => $label): ?>
      <option value="<?php echo $page; ?>" <?php selected($page, $data['page']) ?>><?php echo $label; ?></option>
      <?php endforeach; ?>
    </select>

    <select followAdRules rules="CONFLICTS WITH page BEING 404 AND CONFLICTS WITH page BEING single" rel="place.teaser">
      <?php foreach($ad_context['place.teaser'] as $place => $label): ?>
      <option value="<?php echo $place; ?>" <?php selected($place, $data['place']) ?>><?php echo $label; ?></option>
      <?php endforeach; ?>
    </select>

    <select followAdRules rules="DEPENDS ON page BEING single" rel="place.single">
      <?php foreach($ad_context['place.single'] as $place => $label): ?>
      <option value="<?php echo $place; ?>" <?php selected($place, $data['place']) ?>><?php echo $label; ?></option>
      <?php endforeach; ?>
    </select>

    <select followAdRules rules="DEPENDS ON page BEING 404" rel="place.404">
      <?php foreach($ad_context['place.404'] as $place => $label): ?>
      <option value="<?php echo $place; ?>" <?php selected($place, $data['place']) ?>><?php echo $label; ?></option>
      <?php endforeach; ?>
    </select>

    <label for="ad_<?php echo $id; ?>_n">#</label>
    <input rel="n" id="ad_<?php echo $id; ?>_n" type="text" size="2" value="<?php echo intval($data['n']); ?>" followAdRules rules="DEPENDS ON place.teaser BEING before_post OR place.teaser BEING after_post OR place.single BEING before_comment" />

    <label for="ad_<?php echo $id; ?>_time"><?php _ae("When:"); ?></label>
    <select followAdRules rules="DEPENDS ON page BEING single" rel="when" id="ad_<?php echo $id; ?>_time">
      <?php foreach($ad_context['when'] as $time => $label): ?>
      <option value="<?php echo $time; ?>" <?php selected($time, $data['time']) ?>><?php echo $label; ?></option>
      <?php endforeach; ?>
    </select>

    <label for="ad_<?php echo $id; ?>_to"><?php _ae("To:"); ?></label>
    <select followAdRules rel="to" id="ad_<?php echo $id; ?>_to">
      <?php foreach($ad_context['to'] as $user => $label): ?>
      <option value="<?php echo $user; ?>" <?php selected($user, $data['to']) ?>><?php echo $label; ?></option>
      <?php endforeach; ?>
      <?php foreach($wp_roles->role_names as $role => $label): ?>
      <option value="<?php echo $role; ?>"  <?php selected($role, $data['to']) ?>><?php echo $label; ?></option>
      <?php endforeach; ?>
    </select>
  </p>
  <input name="advertisments[<?php echo $id; ?>][atts]" type="hidden" value="<?php echo $ad['atts']; ?>" />
  <p class="alignright"><input class="button-secondary alignright remove-ad" type="button" value="<?php _ae("Remove"); ?>" followAdRules /></p>
  <p><textarea rows="4" class="code widefat" id="ad_html_<?php echo $id; ?>" name="advertisments[<?php echo $id; ?>][html]"><?php echo $ad['html']; ?></textarea></p>
  <script type="text/javascript">
   /*<![CDATA[*/

   jQuery(document).ready(function($){
     $("#ad_<?php echo $id; ?> [followAdRules]").setupDependencies({disable_only:false, clear_inactive:true, identify_by:'rel'});

     $("#ad_<?php echo $id; ?>").each(function(){
       var block = this;
       //$(this).stop().css("background-color", "#FFFF99").animate({backgroundColor: "#EEEEEE"}, 1500);
       $("[rel]", this).bind('change input', function(){
         var serializedData = $("[rel]:not(:hidden)", block).map(function(i, el){
           var key = $(el).attr('rel').split('.');
           return key[0] + '=' + $(el).val();
         }).get();

         $('[name="advertisments[<?php echo $id; ?>][atts]"]', block).val(serializedData.join('&'));
         return true;
       }).change();
     });

     // map() above may not fire if block is animated/hidden
     setTimeout(function(){$("#ad_<?php echo $id; ?> [rel]").change();}, 2000);
     $("#ad_<?php echo $id; ?> .remove-ad").click(function(){
        $("#ad_<?php echo $id; ?>").animate({
                    "opacity": "hide",
                    "height": "hide",
                    "marginTop": "hide",
                    "marginBottom": "hide",
                    "paddingTop": "hide",
                    "paddingBottom": "hide"
                  }, 333, function(){ $(this).remove(); });
     });
   });
   /*]]>*/
  </script>
 </div>
 <?php
}



/**
 * Add "Featured" column title to the appropriate page (this function is hooked to posts/media column title)
 *
 * @since 1.3
 *
 * @param array $defaults default columns
 */
function atom_setup_featured_column_title($defaults){
  $defaults['featured'] = _a('Featured'); // featured posts
  return $defaults;
}



/**
 * Setup featured column content for Posts (edit.php) page
 *
 * @since 1.0
 *
 * @param string $column_name Current Column
 * @param string $id Post ID
 */
function atom_setup_posts_column_content($column_name, $id){
  if ($column_name == 'featured'):
    $posts = explode(',', atom_get_options('featured_posts'));
    echo '<div class="featured">';
    echo '<a '.(current_user_can('edit_posts') ? 'href="#" rel="post" id="featured-'.$id.'"' : null).' class="'.(in_array($id, $posts) ? "on" : "off").'"></a></div>';
    echo '</div>';
  endif;
}



/**
 * Setup featured column content for Media Library (upload.php) page
 *
 * @since 1.3
 *
 * @param string $column_name Current Column
 * @param string $id Post ID
 */
function atom_setup_media_column_content($column_name, $id){
  if ($column_name == 'featured' && strpos(get_post_mime_type($id), 'image/') !== false): // images only
    $posts = explode(',', get_option('featured_gallery'));
    echo '<div class="featured">';
    echo '<a '.(current_user_can('edit_posts') ? 'href="#" rel="attachment" id="featured-'.$id.'"' : null).' class="'.(in_array($id, $posts) ? "on" : "off").'"></a></div>';
    echo '</div>';
  endif;
}



/**
 * Process the featured content actions (AJAX)
 *
 * @since 1.0
 * @todo Add User Role for managing Featured Posts
 */
function atom_process_featured(){
  check_ajax_referer("theme-settings");
  if(!current_user_can('edit_posts')) return false;

  // read submitted information
  $id = intval($_GET['id']);
  $is_on = (bool)$_GET['isOn'];
  $what = ($_GET['what'] == 'post') ? 'featured_posts' : 'featured_gallery';

  $featured = atom_get_options($what) ? explode(',', atom_get_options($what)) : array();

  // add to array if not on and not currently in the array
  if (!$is_on && ! in_array($id, $featured)) array_push($featured, $id);
  rsort($featured);

  $new_entries = '';

  // if not the same as selected, add to the featured str
  foreach ($featured as $entry)
    if(!($is_on && $entry == $id)) $new_entries .= "{$entry},";

  if($new_entries) $new_entries = substr($new_entries, 0, -1);
  update_option($what, $new_entries);

  atom($what == 'post' ? 'feature_posts' : 'feature_galleries');

  // reverse classes
  exit($is_on ? 'off' : 'on');
}

?>