<?php
/**
 * Theme specific set up.
 * Functions listed here can be defined in the child/parent theme's functions.php file, which is loaded before this script
 *
 * @link http://digitalnature.ro
 *
 * @package ATOM
 * @subpackage Template
 */




/**
 * Active theme settings and their default values
 * In case of a atom_get_options call, this array will be checked first; if no match is found get_option() will be used
 *
 * @since 1.0
 *
 * @return array The theme settings
 */
if(!function_exists('atom_default_settings')){
  function atom_default_settings(){
    return array(

      // internal, always required
      'theme_version' => THEME_VERSION,

      // layout settings -- if any of these is enabled, then all must be enabled as well since they are all connected
      // note that only 'layout' is used for reference
      'layout' => 'col-2-right',
      'page_width' => 'fixed',
      'page_width_max' => 1200,
      'dimensions_fixed_col-2-left' => '320',
      'dimensions_fixed_col-2-right' => '640',
      'dimensions_fixed_col-3' => '240;720',
      'dimensions_fixed_col-3-left' => '240;480',
      'dimensions_fixed_col-3-right' => '480;720',
      'dimensions_fluid_col-2-left' => '30',
      'dimensions_fluid_col-2-right' => '70',
      'dimensions_fluid_col-3' => '25;75',
      'dimensions_fluid_col-3-left' => '25;50',
      'dimensions_fluid_col-3-right' => '50;75',

      'color_scheme' => 'light.css',

      // favicon path
      'favicon' => THEME_URL.'/sample/favicon.ico',

      // logo image path
      'logo' => '',

      // background image/color
      'background_image' => '',
      'background_color' => '000000',

      // css elements on which background image/color is applied - internal settings, required if the above 2 are present
      'background_image_selector' => 'body',
      'background_color_selector' => 'body',

      // footer
      'footer_content' => '<p> [credit] | [link rss] </p>',

      // social media links
      'media_rss' => get_bloginfo('rss2_url'),
      'media_twitter' => 'http://twitter.com/tweetnorris/',
      'media_facebook' => 'http://www.facebook.com/WordPress/',
      'media_flickr' => '',
      'media_myspace' => '',

      // post previews (home, archive, search pages)
      'post_title' => true,
      'post_date' => true,
      'post_date_mode' => 'relative',
      'post_category' => true,
      'post_tags' => true,
      'post_author' => true,
      'post_comments' => true,
      'post_content' => true,
      'post_content_mode' => '50',
      'post_thumbs' => true,
      'post_thumbs_mode' => 'left',
      'post_thumb_size' => '140x140',
      'post_thumb_auto' => true,

      // single post pages
      'single_links' => true,
      'single_print' => true,
      'single_meta' => true,
      'single_share' => true,
      'single_author' => false,
      'single_related' => true,

      // comments
      'comment_flags' => true,
      'comment_filter' => false, // atom_count('all', 'comment') > (atom_count('all', 'post') * 10) ? true : false
      'comment_karma' => true,
      'comment_bury_limit' => -5,

      // ad blocks
      'advertisments' => array(),

       // user css
      'css' => '',

      // advanced
      'jquery' => true,
      'jquery_init_query' => true,
      'optimize' => true,
      'generate_thumbs' => true,
      'lightbox' => true,
      'debug' => false,
      'meta_description' => true,
      'functions' => "<?php\n\n\n",
      'remove_settings' => false,

    );
  }
}


/**
 * Set up custom menus
 *
 * @since 1.2
 *
 * @return array Menus
 */
if(!function_exists('atom_nav_menus')){
  function atom_nav_menus(){
    return array(
      'top' => _a('Page Top'),
      'primary' => _a('Below Header (Main)'),
      'footer' => _a('Above Footer'),
    );
  }
}


/**
 * Set up widgetized areas
 * 'arbitrary' should always be present, otherwhise the [widget] shortcode and tabbed widgets won't work...
 *
 * @since 1.2
 *
 * @return array Widget Areas
 */
if(!function_exists('atom_widget_areas')){
  function atom_widget_areas(){
    return array(

     array(
      'name' => _a('Primary Sidebar'),
      'id' => 'sidebar-1',
      'description' => _a('This is the default sidebar, active on 2 or 3 column layouts. If no widgets are visible, the page will fall back to a single column layout.'),
      'before_widget' => '<li class="block"><div class="block-content block-%2$s clear-block" id="instance-%1$s">',
      'after_widget' => '</div></li>',
      'before_title' => '<h3 class="title"><span>',
      'after_title' => '</span></h3>'
     ),

     array(
      'name' => _a('Secondary Sidebar'),
      'id' => 'sidebar-2',
      'description' => _a('This sidebar is active only on a 3 column setup, if at least one of its widgets is visible to the current user.'),
      'before_widget' => '<li class="block"><div class="block-content block-%2$s clear-block" id="instance-%1$s">',
      'after_widget' => '</div></li>',
      'before_title' => '<h3 class="title"><span>',
      'after_title' => '</span></h3>'
     ),

     array(
      'name' => _a('Primary Footer'),
      'id' => 'footer-1',
      'description' => _a('Active only if at least one of its widgets is visible to the current user. You can add between 1 and 6 widgets here (3 or 4 are optimal). They will adjust their size based on the widget count.'),
      'before_widget' => '<li class="block block-%2$s" id="instance-%1$s"><div class="block-content clear-block">',
      'after_widget' => '</div></li>',
      'before_title' => '<h4 class="title">',
      'after_title' => '</h4>'
     ),

     array(
      'name' => _a("Secondary Footer"),
      'id' => 'footer-2',
      'description' => _a('Active only if at least one of its widgets is visible to the current user.'),
      'before_widget' => '<li class="block block-%2$s" id="instance-%1$s"><div class="block-content clear-block">',
      'after_widget' => '</div></li>',
      'before_title' => '<h4 class="title">',
      'after_title' => '</h4>'
     ),

     array(
      'name' => _a('Arbitrary Widgets'),
      'id' => 'arbitrary',
      'description' => sprintf(_a('Widgets from this area can be grouped into tabs, or added into posts/pages using the %1$s or %2$s shortcodes.'), '[widget ID]', '[widget Name]'),
      'before_widget' => '<div class="block"><div class="block-content block-%2$s clear-block" id="instance-%1$s">',
      'after_widget' => '</div></div>',
      'before_title' => '<h3 class="title"><span>',
      'after_title' => '</span></h3>'
     ),

     array(
      'name' => _a('Featured Content (Header)'),
      'id' => 'featured',
      'description' => _a('This area is designed to incorporate a single featured content widget, such as galleries or featured posts.'),
      'before_widget' => '<div class="featured block-%2$s" id="instance-%1$s">',
      'after_widget' => '</div>',
      'before_title' => '<h4 class="title">',
      'after_title' => '</h4>'
     ),

    );
  }
}



/**
 * Set up widgets
 *
 * @since 1.3
 *
 * @return array Widget IDs
 */
if(!function_exists('atom_widgets')){
  function atom_widgets(){
    return array(
      'archives',
      'authors',
      'blogs',
      'calendar',
      'featured-gallery',
      'featured-posts',
      'recent-comments',
//      'lifestream',
      'links',
      'login',
      'menu',
      'pages',
      'posts',
      'splitter',
      'tabs',
      'tag-cloud',
      'terms',
      'text',
      'top-commenters',
      'twitter',
    );
  }
}



/**
 * Advertisment block contexts
 *
 * @since 1.3
 *
 * @return array
 */
if(!function_exists('atom_ad_context')){
  function atom_ad_context(){
    return array(
      'pages' =>array(
        'home' => _a('Blog Homepage'),
        'single' => _a('Single Post Pages'),
        'category' => _a('Category Archives'),
        'tag' => _a('Tag Archives'),
        'author' => _a('Author Archives'),
        'date' => _a('Date-based Archives'),
        'search' => _a('Search Results'),
        '404' => _a('404 Pages'),
      ),

      'place.single' =>array(
        'before_main_content' => _a("After Header"),
        'before_primary' => _a("Before Main"),
        'after_primary' => _a("After Main"),
        'before_comment' => _a("Before Comment"),
      ),

      'place.teaser' =>array(
        'before_main_content' => _a("After Header"),
        'before_primary' => _a("Before Main"),
        'after_primary' => _a("After Main"),
        'before_post' => _a("Before Post"),
        'after_post' => _a("After Post"),
      ),

      'place.404' =>array(
        'before_primary' => _a("Before 404 message"),
        'after_primary' => _a("After 404 message"),
      ),

      'when' =>array(
        '0' => _a("Any time"),
        'closed' => _a("Comments are closed"),
        'open' => _a("Comments are open"),
      ),

      'to' =>array(
        '0' => _a("Anyone"),
        '1' => _a("Visitor only"),
      ),
    );
  }
}