<?php

/**
 * Archives
 * Differences to the default one:
 *  + Archive type selection
 *  + Limit # of results
 *
 * @since 1.0
 */
 
 
unregister_widget('WP_Widget_Archives'); 
register_widget('atom_widget_archives');  
 
class atom_widget_archives extends WP_Widget {

  function atom_widget_archives() {
    $widget_ops = array('classname' => 'archives', 'description' => _a("Archives of your site's posts"));
    $this->WP_Widget('atom-archives', _a('Archives'), $widget_ops);
  }

  function widget($args, $instance){
    extract($args);
    $instance = wp_parse_args($instance, $this->defaults());
    $title = apply_filters('widget_title', $instance['title'], $instance, $this->id_base);

    $query_args = array(
      'type' => $instance['type'],
      'limit' => intval($instance['limit']),
      'show_post_count' => $instance['count'] ? '1' : '0');

    echo $before_widget;
    if ($title) echo $before_title.$title.$after_title;
    ?>
    <div class="box">

    <?php
    if ($instance['dropdown']):
      $query_args['format'] = 'option'; ?>
      <select class="wide" name="archive-dropdown" onchange='document.location.href=this.options[this.selectedIndex].value;'>
       <option value=""><?php echo esc_attr(_a('Select Month')); ?></option>
       <?php wp_get_archives(apply_filters('widget_archives_dropdown_args', $query_args)); ?>
      </select>
    <?php else: ?>
      <ul> <?php wp_get_archives(apply_filters('widget_archives_args', $query_args)); ?> </ul>
    <?php
    endif;
    ?>

    </div>
    <?php

    echo $after_widget;
  }

  function update($new_instance, $old_instance){
    $instance = $old_instance;
    $instance['title'] = esc_attr($new_instance['title']);
    $instance['type'] = esc_attr($new_instance['type']);
    $instance['count'] = (bool)$new_instance['count'];
    $instance['dropdown'] = (bool)$new_instance['dropdown'];
    $instance['limit'] = intval($new_instance['limit']);
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_archives_defaults', array(
      'title' => _a("Archives"),
      'type' => 'monthly',
      'count' => 0,
      'limit' => 12,
      'dropdown' => '',
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>
    <div <?php atom_widget_form_class($this); ?>>
    <p>
     <label for="<?php echo $this->get_field_id('title'); ?>"><?php _ae('Title:'); ?></label>
     <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($instance['title']); ?>" />
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('type'); ?>"><?php _ae('Type:'); ?></label>
     <select class="widefat" id="<?php echo $this->get_field_id('type'); ?>" name="<?php echo $this->get_field_name('type'); ?>">
      <option value="yearly" <?php selected('yearly', esc_attr($instance['type'])) ?>><?php _ae("Yearly"); ?></option>
      <option value="monthly" <?php selected('monthly', esc_attr($instance['type'])) ?>><?php _ae("Monthly"); ?></option>
      <option value="weekly" <?php selected('weekly', esc_attr($instance['type'])) ?>><?php _ae("Weekly"); ?></option>
      <option value="daily" <?php selected('daily', esc_attr($instance['type'])) ?>><?php _ae("Daily"); ?></option>
     </select>
    </p>

    <p>
     <input type="checkbox" <?php checked(isset($instance['count']) ? (bool)$instance['count'] : false); ?> id="<?php echo $this->get_field_id('count'); ?>" name="<?php echo $this->get_field_name('count'); ?>" />
     <label for="<?php echo $this->get_field_id('count'); ?>"><?php _ae('Show post count'); ?></label>

     <br />

     <input type="checkbox" <?php checked(isset($instance['dropdown']) ? (bool)$instance['dropdown'] : false); ?> id="<?php echo $this->get_field_id('dropdown'); ?>" name="<?php echo $this->get_field_name('dropdown'); ?>" />
     <label for="<?php echo $this->get_field_id('dropdown'); ?>"><?php _ae('Show as dropdown'); ?></label>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('limit'); ?>"><?php _ae('Limit to:'); ?></label><br />
     <input size="5" id="<?php echo $this->get_field_id('limit'); ?>" name="<?php echo $this->get_field_name('limit'); ?>" type="text" value="<?php echo intval($instance['limit']); ?>" /> <small><em><?php _ae("Archives"); ?></em></small>
    </p>
    </div>
    <?php
  }
}