<?php

/**
 * A list of Authors
 *
 * @since 1.0
 * @todo add feed link option
 * @todo add fade to image links in avatar-only mode
 * @todo "show more" link (limit author count)
 */
 
register_widget('atom_widget_authors'); 
 
class atom_widget_authors extends WP_Widget{
  function atom_widget_authors() {
   $widget_ops = array('classname' => 'authors', 'description' => _a("A list of authors from your blog"));
   $control_ops = array('width' => 500);
   $this->WP_Widget('atom-authors', _a('Authors'), $widget_ops, $control_ops);

    // flush cache when posts are changed
    add_action('save_post', array(&$this, 'flush_cache'));
    add_action('deleted_post', array(&$this, 'flush_cache'));
    add_action('switch_theme', array(&$this, 'flush_cache'));
  }

  function sort_by_display_name($a, $b){ return ($a->display_name > $b->display_name); }
  function sort_by_post_count($a, $b){ return ($a->post_count < $b->post_count); }

  function get_authors($sort_by = '') {

    $users = get_users_of_blog();
    $author_ids = array();

    foreach($users as $user) $author_ids[] = $user->user_id;
    $author_post_count = count_many_users_posts($author_ids);
    foreach($users as $user) $user->post_count = $author_post_count[$user->user_id];

    // sort
    usort($users, array(&$this, "sort_by_$sort_by"));

    return $users;
  }

  function flush_cache(){
    wp_cache_delete('widget_authors', 'widget');
  }

  function template($mode = 'full'){
    switch($mode):
      case 'full':
        return "<a class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{TITLE}\">\n"
              ." {AVATAR}\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{NAME}</span>\n"
              ."   <span class=\"c1\">{POST_COUNT}</span>\n"
              ." </span>\n"
              ."</a>\n";
        break;
      case 'images':
        return "<a class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{TITLE}\">\n"
              ." {AVATAR}\n"
              ."</a>\n";
        break;
      case 'brief':
        return "<a class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{TITLE}\">\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{NAME}</span>\n"
              ." </span>\n"
              ."</a>\n";
        break;
      case 'detailed':
        return "<a class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{TITLE}\">\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{NAME}</span>\n"
              ."   <span class=\"c1\">{POST_COUNT}</span>\n"
              ." </span>\n"
              ."</a>\n";
        break;

    endswitch;
  }

  function widget($args, $instance){

    $cache = wp_cache_get('widget_authors', 'widget');
    if (!is_array($cache)) $cache = array();
    if (isset($cache[$args['widget_id']])):
      echo $cache[$args['widget_id']];
      return;
    endif;

    extract($args);
    $instance = wp_parse_args($instance, $this->defaults());
    $title = apply_filters('widget_title', $instance['title'], $instance, $this->id_base);

    $show_avatars = ($instance['mode'] != 'brief' && $instance['mode'] != 'detailed');
    $details = ($instance['mode'] == 'detailed' || $instance['mode'] == 'full');

    $authors = $this->get_authors($instance['sort_by']);
    if(empty($authors)) atom_add_debug_message("No authors found in {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");

    $output = $before_widget;
    if ($title) $output .= $before_title.$title.$after_title;

    $output .= "<ul class=\"menu authors {$instance['mode']}\">";
    $count = 0;
    if($instance['mode'] == 'template') $template = $instance['template']; else $template = $this->template($instance['mode']);
    foreach($authors as $author):
      if($instance['exclude_admin'] && $author->display_name == 'admin') continue;
      if((!$instance['hide_empty']) || ($instance['hide_empty'] && $author->post_count > 0)):
        $count++;
        $output .= '<li'.($count == 1 ? ' class="first"' : NULL).'>';

        if($instance['mode'] == 'images' || $instance['mode'] == 'brief')
          $title = sprintf(_a('%1$s (%2$s posts)'), $author->display_name, $author->post_count);
        else $title = sprintf(_a('Posts by %s'), $author->display_name);

        // output template
        $output .= atom_render_template($template, array(
          'TITLE' => $title,
          'URL' => get_author_posts_url($author->user_id, $author->display_name),
          'AVATAR' => atom_get_avatar($author->user_email, intval($instance['avatar_size']), '', $author->display_name),
          'KARMA' => intval(get_user_meta($author->user_id, "karma", true)),
          'NAME' => $author->display_name,
          'POST_COUNT' => sprintf(_an('%s post', '%s posts', $author->post_count), number_format_i18n($author->post_count)),
          'RSS' => get_author_feed_link($author->user_id),  // not used by default
          'ID' => $author->user_id, // same...
         ));

        $output .= '</li>';
      endif;
    endforeach;
    $output .= '</ul>';

    $output .= $after_widget;

    echo $output;
    $cache[$args['widget_id']] = $output;
    wp_cache_set('widget_authors', $cache, 'widget');
  }

  function update($new_instance, $old_instance){
    $instance['title'] = esc_attr($new_instance['title']);
    $instance['mode'] = esc_attr($new_instance['mode']);
    $instance['hide_empty'] = (bool)$new_instance['hide_empty'];
    $instance['sort_by'] = esc_attr($new_instance['sort_by']);
    $instance['exclude_admin'] = (bool)$new_instance['exclude_admin'];
    $instance['avatar_size'] = intval($new_instance['avatar_size']);

    // html template
    if(isset($new_instance['template']) && current_user_can('edit_themes')) $instance['template'] = $new_instance['template'];

    $this->flush_cache();
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    return apply_filters('atom_widget_authors_defaults', array(
      'title' => _a("Authors"),
      'mode' => 'full',
      'avatar_size' => 48,
      'sort_by' => 'post_count',
      'hide_empty' => true,
      'exclude_admin' => false,
      'template' => $this->template('full'),
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>
    <div <?php atom_widget_form_class($this); ?>>
    <p>
      <label for="<?php echo $this->get_field_id('title'); ?>"><?php _ae('Title:') ?></label>
      <input type="text" class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" value="<?php if (isset($instance['title'])) echo esc_attr($instance['title']); ?>" />
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('avatar_size'); ?>"><?php _ae('Avatar Size:') ?></label>
      <input type="text" size="3" id="<?php echo $this->get_field_id('avatar_size'); ?>" name="<?php echo $this->get_field_name('avatar_size'); ?>" value="<?php if (isset($instance['avatar_size'])) echo esc_attr($instance['avatar_size']); ?>" /> <?php _ae("pixels"); ?>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('sort_by'); ?>"><?php _ae('Sort by') ?></label>
      <select id="<?php echo $this->get_field_id('sort_by'); ?>" name="<?php echo $this->get_field_name('sort_by'); ?>">
       <option value="display_name" <?php selected($instance['sort_by'], "display_name"); ?>><?php _ae("Name"); ?></option>
       <option value="post_count" <?php selected($instance['sort_by'], "post_count"); ?>><?php _ae("Post Count"); ?></option>
      </select>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('hide_empty'); ?>">
        <input type="checkbox" id="<?php echo $this->get_field_id('hide_empty'); ?>" name="<?php echo $this->get_field_name('hide_empty'); ?>"<?php checked($instance['hide_empty']); ?> />
        <?php _ae('Only show authors with posts'); ?>
      </label>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('exclude_admin'); ?>">
        <input type="checkbox" id="<?php echo $this->get_field_id('exclude_admin'); ?>" name="<?php echo $this->get_field_name('exclude_admin'); ?>"<?php checked($instance['exclude_admin']); ?> />
	    <?php _ae('Exclude Admin'); ?>
      </label>
    </p>

    <div class="high-priority-block">
    <p>
      <label for="<?php echo $this->get_field_id('mode'); ?>"><strong><?php _ae('Display mode:') ?></strong></label><br />
      <select id="<?php echo $this->get_field_id('mode'); ?>" name="<?php echo $this->get_field_name('mode'); ?>" followRules>
       <option value="images" <?php selected($instance['mode'], "images"); ?>><?php _ae("Avatars only"); ?></option>
       <option value="brief" <?php selected($instance['mode'], "brief"); ?>><?php _ae("Author names only"); ?></option>
       <option value="detailed" <?php selected($instance['mode'], "detailed"); ?>><?php _ae("Detailed Info"); ?></option>
       <option value="full" <?php selected($instance['mode'], "full"); ?>><?php _ae("Detailed Info &amp; Avatars"); ?></option>
       <?php if(current_user_can('edit_themes')): ?>
       <option value="template" <?php selected($instance['mode'], "template"); ?>><?php _ae("Advanced, use template below"); ?></option>
       <?php endif; ?>
      </select>
    </p>
    </div>

    <?php if(current_user_can('edit_themes')): ?>
    <div class="high-priority-block">
      <p>
      <label for="<?php echo $this->get_field_id('template'); ?>"><strong><?php _ae('Entry Template (HTML):'); ?></strong>
      <input type="hidden" name="<?php echo $this->get_field_name('template'); ?>" value="<?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?>" />
      <textarea followRules rules="DEPENDS ON <?php echo $this->get_field_name('mode'); ?> BEING template" class="widefat code" id="<?php echo $this->get_field_id('template'); ?>" name="<?php echo $this->get_field_name('template'); ?>" rows="8" cols="28"><?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?></textarea>
      </label>
      </p>
    </div>
    <?php endif; ?>
    </div>
    <?php
  }
}