<?php

/**
 * Blog list
 *
 * @since 1.3
 */
 
if(atom_mu_network()) register_widget('atom_widget_blogs'); 
 
class atom_widget_blogs extends WP_Widget{

  function atom_widget_blogs(){
    $widget_ops = array('classname' => 'blogs', 'description' => _a('A list of site blogs'));
    $control_ops = array('width' => 500);
    $this->WP_Widget('atom-blogs', _a('Network Blogs'), $widget_ops, $control_ops);

    atom_add('ajax_requests', array(&$this, 'ajax'));

    // include in jQuery(document).ready()
    atom_add('jquery_init', array(&$this, 'js'));

    // flush cache when blog is updated
    add_action('wpmu_blog_updated', array(&$this, 'flush_cache'));
  }

  function flush_cache(){
    wp_cache_delete('widget_blogs', 'widget');
  }

  function js(){
    // we need to process all instances because this function gets to run only once
    $widget_settings = atom_get_options($this->option_name);

    foreach((array)$widget_settings as $instance => $options):

      // identify instance
      $id = "{$this->id_base}-{$instance}";
      $block_id = "instance-{$id}";

      if (isset($options['more']) && $options['more'] && is_active_widget(false, $id, $this->id_base)): ?>

      $("#<?php echo $block_id; ?>").delegate("a.more", "click", function(){
        event.preventDefault();

        $.ajax({
          type: "GET",
          url: "<?php echo home_url(); //echo atom_get_current_page_url(); ?>",
          data: { instance: <?php echo $instance; ?>,
                  offset: parseInt($(this).attr('rel')),
                  _ajax_nonce: "<?php atom_nonce('get_blogs'); ?>",
                  atom: "get_blogs" },
          dataType: 'json',
          context: this,
          beforeSend: function() { $(this).addClass("loading"); },
          complete: function() { $(this).removeClass("loading"); },
          success: function(response){
            var link = $(this);

            // append to list & update tab container height (if we're inside a tabbed widget)
            if(response.output != '')
              $(response.output).appendTo("#<?php echo $block_id; ?> ul").hide().each(function(i){
                 $(this).delay(333*i).animate(
                    {
                      "opacity": "show",
                      "height": "show",
                      "marginTop": "show",
                      "marginBottom": "show",
                      "paddingTop": "show",
                      "paddingBottom": "show"
                    },
                    { duration: 333,
                      step: function(now, fx){
                      link.parents('li.block .sections').height((link.parents('#<?php echo $block_id; ?>').height()) + 5);
                    }
                 });
              });

            link.attr('rel', response.offset);

            // no more data?
            if(!response.more) link.hide();

          }
        });

      });

      <?php endif;
    endforeach;
  }

  function ajax(){
    if(atom_ajax_request('get_blogs')):
      atom_ajax_header('get_blogs', 'application/json');

      $settings = atom_get_options($this->option_name);
      $instance = intval($_GET['instance']);
      $settings = $settings[$instance];

      $offset = isset($_GET['offset']) ? intval($_GET['offset']) : 0;
      $output = $this->get_blogs($settings, $next, $offset);
      $offset = $offset + $settings['number'];

      echo json_encode(array('output' => $output, 'more' => $next, 'offset' => $offset));
      exit();
    endif;
  }

  function template(){
    return "<a class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{TITLE}\">\n"
              ." {AVATAR}\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{TITLE}</span>\n"
              ."   <span class=\"c1\">{DESCRIPTION}</span>\n"
              ."   <span class=\"c2\">Updated {UPDATED}</span>\n"
              ." </span>\n"
              ."</a>\n";
  }

  function get_blogs($args, &$more, $offset = 0){
    extract($args);

    $sites = atom_get_sites(array(
      'exclude_id' => $exclude,
      'exclude_mature' => $exclude_mature,
      'sort_column' => $order_by,
      'order' => ($order_by == 'id') ? 'ASC' : 'DESC',
      'limit_results' => $number + 1,
      'start' => $offset
     ));

    // because mysql rand() is slowwwww
    if($order_by == 'rand') shuffle($sites);

    $more = count($sites) > $number ? true : false;
    $count = 1;
    $output = '';
    foreach((array)$sites as $site):

      if($count++ == $number+1) break;
      $output .= '<li'.($count == 1 && $offset == 0  ? ' class="first"' : NULL).'>';

      // output template
      $output .= atom_render_template($args['template'], array(
        'TITLE' => get_blog_option($site['blog_id'], 'blogname'),
        'URL' => get_blogaddress_by_domain($site['domain'], $site['path']), // should be faster than get_blog_option($site['blog_id'], 'siteurl'),
        'AVATAR' => atom_get_avatar($site['email'], $avatar_size, ''),
        'POST_COUNT' => get_blog_option($site['blog_id'], 'post_count'),
        'DESCRIPTION' => convert_smilies(get_blog_option($site['blog_id'], 'blogdescription')),
        'UPDATED' => atom_time_since(abs(strtotime($site['last_updated']))),
        'REGISTERED' => atom_time_since(abs(strtotime($site['registered']))),
        'EMAIL' => esc_url($site['email']), // should not be used
        'ID' => $site['blog_id'],
       ));

      $output .= '</li>';
    endforeach;

    return $output;
  }

  function widget($args, $instance){

    $cache = wp_cache_get('widget_blogs', 'widget');
    if (!is_array($cache)) $cache = array();
    if (isset($cache[$args['widget_id']])):
      echo $cache[$args['widget_id']];
      return;
    endif;

    extract($args, EXTR_SKIP);
    $instance = wp_parse_args($instance, $this->defaults());

    $blogs = $this->get_blogs($instance, $next);
    if(!$blogs) return atom_add_debug_message("No blogs found in {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");

    $title = apply_filters('widget_title', $instance['title'], $instance, $this->id_base);

    $output = $before_widget;
    if($title) $output .= $before_title.$title.$after_title;

    $output .= "<ul class=\"menu full blogs\">{$blogs}</ul>";
    if($instance['more'] && $next && atom_get_options('jquery'))
      $output .= "<a href=\"#\" class=\"fadeThis more\" rel=\"{$instance['number']}\">"._a("Show More")."</a>";

    $output .= $after_widget;

    echo $output;
    $cache[$args['widget_id']] = $output;
    wp_cache_set('widget_blogs', $cache, 'widget');
  }

  function update($new_instance, $old_instance){
    $instance = $old_instance;
    $instance['title'] = esc_attr($new_instance['title']);
    $instance['order_by'] = esc_attr($new_instance['order_by']);
    $instance['number'] = min(max(intval($new_instance['number']), 1), 20);
    $instance['exclude'] = esc_attr($new_instance['exclude']);
    $instance['exclude_mature'] = (bool)$new_instance['exclude_mature'];
    $instance['avatar_size'] = intval($new_instance['avatar_size']);
    $instance['more'] = (bool)$new_instance['more'];

    // html template
    if(current_user_can('edit_themes')) $instance['template'] = $new_instance['template'];

    $this->flush_cache();
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_blogs_defaults', array(
      'title' => _a("Recently updated blogs"),
      'order_by' => 'last_updated',
      'number' => 10,
      'exclude' => '',
      'exclude_mature' => false,
      'avatar_size' => 48,
      'more' => true,
      'template' => $this->template(),
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>
    <div <?php atom_widget_form_class($this); ?>>

    <p>
     <label for="<?php echo $this->get_field_id('title'); ?>"><?php _ae('Title:'); ?></label>
     <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php if (isset($instance['title'])) echo esc_attr($instance['title']); ?>" />
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('order_by'); ?>"><?php _ae('Order by:') ?></label>
      <select id="<?php echo $this->get_field_id('order_by'); ?>" name="<?php echo $this->get_field_name('order_by'); ?>">
       <option value="last_updated" <?php selected($instance['order_by'], "last_updated"); ?>><?php _ae("Last Update"); ?></option>
       <option value="registered" <?php selected($instance['order_by'], "registered"); ?>><?php _ae("Registered Date"); ?></option>
       <option value="id" <?php selected($instance['order_by'], "id"); ?>><?php _ae("Site ID"); ?></option>
       <option value="rand" <?php selected($instance['order_by'], "rand"); ?>><?php _ae("Nothing, Randomize"); ?></option>
      </select>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('number'); ?>"><?php _ae('How many entries to display?'); ?></label>
     <input id="<?php echo $this->get_field_id('number'); ?>" name="<?php echo $this->get_field_name('number'); ?>" type="text" value="<?php if (isset($instance['number'])) echo intval($instance['number']); ?>" size="3" />
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('exclude'); ?>"><?php _ae('Exclude:'); ?></label> <input type="text" value="<?php echo esc_attr($instance['exclude']); ?>" name="<?php echo $this->get_field_name('exclude'); ?>" id="<?php echo $this->get_field_id('exclude'); ?>" class="widefat" />
      <br />
      <small><?php _ae('Site IDs, separated by commas.'); ?></small>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('exclude_mature'); ?>">
     <input type="checkbox" id="<?php echo $this->get_field_id('exclude_mature'); ?>" name="<?php echo $this->get_field_name('exclude_mature'); ?>"<?php checked($instance['exclude_mature']); ?> />
 	   <?php _ae('Exclude sites marked as mature'); ?></label>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('avatar_size'); ?>"><?php _ae('Avatar Size:') ?></label>
      <input type="text" size="3" id="<?php echo $this->get_field_id('avatar_size'); ?>" name="<?php echo $this->get_field_name('avatar_size'); ?>" value="<?php if (isset($instance['avatar_size'])) echo esc_attr($instance['avatar_size']); ?>" /> <?php _ae("pixels"); ?>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('more'); ?>" <?php if(!atom_get_options('jquery')) echo "class=\"disabled\""; ?>>
     <input <?php if(!atom_get_options('jquery')) echo "disabled=\"disabled\""; ?> type="checkbox" id="<?php echo $this->get_field_id('more'); ?>" name="<?php echo $this->get_field_name('more'); ?>"<?php checked($instance['more']); ?> />
     <?php printf(_a('Display %s Link'), '<code>'._a("Show More").'</code>'); ?></label>
    </p>

    <?php if(current_user_can('edit_themes')): ?>
    <div class="high-priority-block">
      <p>
      <label for="<?php echo $this->get_field_id('template'); ?>"><strong><?php _ae('Entry Template (HTML):'); ?></strong>
      <textarea class="widefat code" id="<?php echo $this->get_field_id('template'); ?>" name="<?php echo $this->get_field_name('template'); ?>" rows="8" cols="28"><?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?></textarea>
      </label>
      </p>
    </div>
    <?php endif; ?>
    </div>
  <?php
  }
}