<?php
/**
 * Featured Gallery
 *
 * @link http://digitalnature.ro
 *
 * @package ATOM
 * @subpackage Template
 *
 *
 * @since 1.3
 *
 * @todo implement nextgen gallery bridge
 * @todo add ajax option (request 2nd image and so on...)
 */



register_widget('atom_widget_featured_gallery');

class atom_widget_featured_gallery extends WP_Widget{

  function atom_widget_featured_gallery(){
    $widget_ops = array('classname' => 'featured-gallery', 'description' => _a('Photo gallery showcase'));
    $control_ops = array('width' => 500);
    $this->WP_Widget('atom-featured-gallery', _a('Featured Gallery'), $widget_ops, $control_ops);

    // include in jQuery(document).ready()
    atom_add('jquery_init', array(&$this, 'js'));

    atom_add('feature_galleries', array(&$this, 'flush_cache'));
    //add_action('save_post', array(&$this, 'flush_cache'));
    //add_action('deleted_post', array(&$this, 'flush_cache'));
  }

  function flush_cache(){
    wp_cache_delete('widget_featured_gallery', 'widget');
  }

  function js(){
    // we need to process all instances because this function gets to run only once
    $widget_settings = atom_get_options($this->option_name);

    foreach((array)$widget_settings as $instance => $options):

      // identify instance
      $id = "{$this->id_base}-{$instance}";
      $block_id = "instance-{$id}";

      if(is_active_widget(false, $id, $this->id_base)): ?>

       $("#<?php echo $block_id; ?> .featured-gallery .cycle").cycle({
         fx: '<?php echo $options['effect']; ?>',
         speed: <?php echo ($options['effect'] != 'toss') ? '500' : '2000'; ?>,
         pause: true,
         prev: '#<?php echo $block_id; ?> .featured-gallery a.prev',
         next: '#<?php echo $block_id; ?> .featured-gallery a.next',
         easing: '<?php echo $options['easing']; ?>',
         timeout:  <?php echo abs(intval($options['delay'])) * 1000; ?>
       });

       <?php if($options['caption']): ?>
       $('#<?php echo $block_id; ?> .featured-gallery li').hover(function(){
         $('.caption', this).stop().animate({bottom:'0px',opacity:1},{queue:false,duration:200,easing:'easeOutExpo'});
       }, function() {
         $('.caption', this).stop().animate({bottom:'-70px',opacity:0},{queue:false,duration:300,easing:'easeInExpo'});
       });
       <?php endif; ?>

      <?php endif;
    endforeach;
  }

  function get_image($id, $captions = true, $link = 'image', $size = 'gallery') {
    $image = wp_get_attachment_image_src($id, $size);
    if(!$image) return '';

    $html = '';
    list($src, $width, $height) = $image;
    $hwstring = image_hwstring($width, $height);
    if(is_array($size)) $size = join('x', $size);
    $attachment = &get_post($id);

    $alt = trim(strip_tags(get_post_meta($id, '_wp_attachment_image_alt', true)));
    if(empty($alt)) $alt = trim(strip_tags($attachment->post_excerpt));
    if(empty($alt)) $alt = trim(strip_tags($attachment->post_title));

    $attr = array(
      'src'   => $src,
      'class' => "photo",
      'alt'   => $alt,
    );

    // show titles only if captions are disabled
    if(!$captions) $attr['title'] = trim(strip_tags($attachment->post_title));

    $attr = apply_filters('wp_get_attachment_image_attributes', $attr, $attachment);
    $attr = array_map('esc_attr', $attr);
    $html = rtrim("<img $hwstring");
    foreach($attr as $name => $value) $html .= " {$name}=\"{$value}\"";
    $html .= ' />';

    $caption = empty($attachment->post_content) ? $attachment->post_excerpt : $attachment->post_content;

    if($link == 'post' && ($parent = get_post_field('post_parent', $id))) $link = get_permalink($parent);
    elseif($link == 'image' && is_array($src = wp_get_attachment_image_src($id, 'full'))) $link = $src[0];
    else $link = '';

    if($link) $html = '<a href="'.$link.'" title="'.strip_tags($caption).'" rel="'.$this->id.'">'.$html.'</a>';

    if($captions && !empty($caption))
      $html .= '<div class="caption"><p>'.atom_filter_content($caption, array(
        'limit' => 50,
        'allowed_tags' => array('a', 'abbr', 'acronym', 'b', 'cite', 'code', 'del', 'dfn', 'em', 'i', 'ins', 'q', 'strong', 'sub', 'sup'),
        'more' => '[...]',
      )).'</p></div>';

    return $html;
  }

  function get_featured_media($args){
    extract($args);
    $media_ids = explode(',', get_option('featured_gallery'));
    shuffle($media_ids);
    $items = array();
    $count = 1;
    foreach($media_ids as $media):
      $image = $this->get_image($media, $caption, $link);
      if($image) $items[] = $image;
      if($count++ > $number) break;
    endforeach;

    return empty($items) ? false : $items;
  }

  function widget($args, $instance){
    $cache = wp_cache_get('widget_featured_gallery', 'widget');
    if (!is_array($cache)) $cache = array();
    if (isset($cache[$args['widget_id']])):
      echo $cache[$args['widget_id']];
      return;
    endif;

    extract($args, EXTR_SKIP);
    $instance = wp_parse_args($instance, $this->defaults());

    $images = $this->get_featured_media($instance);
    if(!$images) return atom_add_debug_message("No relevant gallery images found {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");

    ob_start();
    echo $before_widget;
    ?>

    <!-- featured gallery -->
    <div class="featured-gallery">
     <div class="block-content clear-block">

       <?php if((count($images) > 1) && $instance['navigation']): // enable navigation only if we have more than 5 posts ?>
       <a class="prev">&laquo;</a>
       <a class="next">&raquo;</a>
       <?php endif; ?>

       <ul class="items <?php if(count($images) > 1) echo "cycle"; ?>">
          <?php foreach ($images as $image): ?>
          <li class="item">
            <?php echo $image; ?>
          </li>
          <?php endforeach; ?>
       </ul>
     </div>
    </div>
    <!-- /featured gallery -->

    <?php
    echo $after_widget;
    $output = ob_get_clean();

    echo $output;
    $cache[$args['widget_id']] = $output;
    wp_cache_set('widget_featured_gallery', $cache, 'widget');
  }

  function update($new_instance, $old_instance){
    $instance = $old_instance;
    $instance['source'] = esc_attr($new_instance['source']);
    $instance['nextgen'] = intval($new_instance['nextgen']);
    $instance['link'] = esc_attr($new_instance['link']);
    $instance['number'] = min(max(intval($new_instance['number']), 1), 20);                                         
    $instance['caption'] = (bool)$new_instance['caption'];
    $instance['navigation'] = (bool)$new_instance['navigation'];
    $instance['timeframe'] = intval($new_instance['timeframe']);
    $instance['delay'] = intval($new_instance['delay']);
    $instance['effect'] = esc_attr($new_instance['effect']);
    $instance['easing'] = esc_attr($new_instance['easing']);

    $this->flush_cache();
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_featured_gallery_defaults', array(
      'source' => 'marked',
      'nextgen' => 0,
      'link' => 'image',
      'number' => 6,
      'caption' => true,
      'navigation' => false,
      'timeframe' => 60,
      'delay' => 10,
      'effect' => 'fade',
      'easing' => 'easeOutCirc',

      // hidden options
      'order' => 'rand',
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>

    <?php if(!atom_get_options('jquery')): ?>
      <div <?php atom_widget_form_class($this); ?>><p class="widget-error"><?php _ae("This widget requires jQuery to be enabled"); ?></p></div>
      <?php return;
    endif;?>

    <div <?php atom_widget_form_class($this); ?>>

    <div class="high-priority-block">

      <p><strong><em><?php _ae("Content Source:"); ?></em></strong></p>
      <label for="<?php echo $this->get_field_id('source'); ?>_marked">
        <input type="radio" value="marked" id="<?php echo $this->get_field_id('source'); ?>_marked" name="<?php echo $this->get_field_name('source'); ?>" <?php checked($instance['source'], 'marked'); ?> />
        <?php printf(_a("%s that I mark as featured"), '<a href="'.admin_url('edit.php').'">'._a("Media items").'</a>'); ?>
      </label>
      <br />
      <label for="<?php echo $this->get_field_id('source'); ?>_nextgen" class="disabled">
        <input disabled="disabled" id="<?php echo $this->get_field_id('source'); ?>_nextgen" name="<?php echo $this->get_field_name('source'); ?>" type="radio" value="nextgen" <?php checked($instance['source'], 'nextgen'); ?> />
        <?php _ae("NextGEN gallery"); ?>
        <select disabled="disabled">
          <option><?php _ae("-- All galleries --"); ?></option>
        </select>
      </label>
    </div>

    <p>
      <label for="<?php echo $this->get_field_id('link'); ?>"><?php _ae('Link to:') ?></label>
      <select id="<?php echo $this->get_field_id('link'); ?>" name="<?php echo $this->get_field_name('link'); ?>">
       <option value="image" <?php selected($instance['link'], 'image'); ?>><?php _ae("Original image size"); ?></option>
       <option value="post" <?php selected($instance['link'], 'post'); ?>><?php _ae("Attached post, if any"); ?></option>
       <option value="" <?php selected($instance['link'], ''); ?>><?php _ae("Nothing"); ?></option>
      </select>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('number'); ?>"><?php _ae('How many entries to display?'); ?></label>
     <input id="<?php echo $this->get_field_id('number'); ?>" name="<?php echo $this->get_field_name('number'); ?>" type="text" value="<?php if (isset($instance['number'])) echo intval($instance['number']); ?>" size="3" />
    </p>

    <p>
     <?php $input = '<input id="'.$this->get_field_id('timeframe').'" name="'.$this->get_field_name('timeframe').'" type="text" value="'.(isset($instance['timeframe']) ? intval($instance['timeframe']) : null).'" size="3" />'; ?>
     <label for="<?php echo $this->get_field_id('timeframe'); ?>"><?php printf(_a('Exclude content older than %s day(s)'), $input); ?></label>
    </p>

    <p>
     <?php $input = '<input id="'.$this->get_field_id('delay').'" name="'.$this->get_field_name('delay').'" type="text" value="'.(isset($instance['delay']) ? intval($instance['delay']) : null).'" size="3" />'; ?>
     <label for="<?php echo $this->get_field_id('delay'); ?>"><?php printf(_a('Cycle every %s second(s)'), $input); ?></label>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('caption'); ?>">
     <input type="checkbox" id="<?php echo $this->get_field_id('caption'); ?>" name="<?php echo $this->get_field_name('caption'); ?>"<?php checked($instance['caption']); ?> />
 	   <?php _ae('Show Captions (if description is available)'); ?></label>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('navigation'); ?>">
     <input type="checkbox" id="<?php echo $this->get_field_id('navigation'); ?>" name="<?php echo $this->get_field_name('navigation'); ?>"<?php checked($instance['navigation']); ?> />
 	   <?php _ae('Show Previous/Next Links'); ?></label>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('effect'); ?>"><?php _ae('Transition effect:') ?></label>
      <select id="<?php echo $this->get_field_id('effect'); ?>" name="<?php echo $this->get_field_name('effect'); ?>">
       <?php foreach(unserialize(EFFECTS) as $fx => $name): ?>
       <option value="<?php echo $fx; ?>" <?php selected($instance['effect'], $fx); ?>><?php echo $name; ?></option>
       <?php endforeach; ?>
      </select>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('easing'); ?>"><?php _ae('Easing:') ?></label>
      <select id="<?php echo $this->get_field_id('easing'); ?>" name="<?php echo $this->get_field_name('easing'); ?>">
       <?php foreach(unserialize(EASING) as $fx => $name): ?>
       <option value="<?php echo $fx; ?>" <?php selected($instance['easing'], $fx); ?>><?php echo $name; ?></option>
       <?php endforeach; ?>
      </select>
    </p>

    </div>
  <?php
  }
}