<?php

/**
 * Featured Posts
 *
 * @since 1.3
 * @todo find a way to make this fluid when fluid layout is used
 */
 
register_widget('atom_widget_featured_posts');  
 
class atom_widget_featured_posts extends WP_Widget{

  function atom_widget_featured_posts(){
    $widget_ops = array('classname' => 'featured-posts', 'description' => _a('Slides featured posts'));
    $control_ops = array('width' => 500);
    $this->WP_Widget('atom-featured-posts', _a('Featured Posts'), $widget_ops, $control_ops);

    // include in jQuery(document).ready()
    atom_add('jquery_init', array(&$this, 'js'));

    atom_add('feature_posts', array(&$this, 'flush_cache'));
    add_action('save_post', array(&$this, 'flush_cache'));
    add_action('deleted_post', array(&$this, 'flush_cache'));
  }

  function flush_cache(){
    wp_cache_delete('widget_featured_posts', 'widget');
  }

  function js(){
    // we need to process all instances because this function gets to run only once
    $widget_settings = atom_get_options($this->option_name);

    foreach((array)$widget_settings as $instance => $options):

      // identify instance
      $id = "{$this->id_base}-{$instance}";
      $block_id = "instance-{$id}";

      if(is_active_widget(false, $id, $this->id_base)): ?>

       $("#<?php echo $block_id; ?> .featured-posts .cycle").each(function(index, value){
         var count = (index + 1) * 150;
         var list = $(this),
             featured_cycle = function(){
               list.cycle({
                 fx: '<?php echo $options['effect']; ?>',
                 speed: <?php echo ($options['effect'] != 'toss') ? '500' : '2000'; ?>,
                 pause: true,
                 prev: '#<?php echo $block_id; ?> .block-featured a.prev',
                 next: '#<?php echo $block_id; ?> .block-featured a.next',
                 easing: '<?php echo $options['easing']; ?>',
                 timeout:  <?php echo abs(intval($options['delay'])) * 1000; ?>
               });
             };

         setTimeout(featured_cycle, count);
       });

       $('#<?php echo $block_id; ?> .featured-posts li').hover(function(){
         $('.caption', this).stop().animate({bottom:'0px',opacity:1},{queue:false,duration:200,easing:'easeOutExpo'});
       }, function() {
         $('.caption', this).stop().animate({bottom:'-70px',opacity:0},{queue:false,duration:300,easing:'easeInExpo'});
       });



      <?php endif;
    endforeach;
  }

  function template(){
    return "<a class=\"post-thumb\" href=\"{URL}\" title=\"{TITLE}\">\n"
          ."  {THUMBNAIL}\n"
          ."</a>\n";
  }

  // by azspot at gmail dot com --  http://www.php.net/manual/en/function.array-chunk.php#75022
  function array_chunk_fixed($list, $parts){
    $total_length = count($list);
    $part_length = floor($total_length / $parts);
    $part_rem = $total_length % $parts;
    $output = array();
    $mark = 0;
    for ($i = 0; $i < $parts; $i++):
      $inc = ($i < $part_rem) ? $part_length + 1 : $part_length;
      $output[$i] = array_slice($list, $mark, $inc);
      if(empty($output[$i])) unset($output[$i]);
      $mark += $inc;
    endfor;
    return $output;
  }

  function get_featured_posts($args){
    global $post;

    extract($args);

    // restrict to a specific date range
    // based on Austin Matzko's code from wp-hackers email list
    $time = date('Y-m-d', strtotime('-'.intval($timeframe).' days'));
    $where = create_function('$a', 'return $a.'.'"'." AND post_date > '$time'".'"'.';');
    add_filter("posts_where", $where);

    $posts = new WP_Query();
    $query = array('cat' => ($source == 'category') ? intval($category) : false,
                   'post__in' => ($source == 'marked') ? explode(',', atom_get_options('featured_posts')) : false,
                   'caller_get_posts' => 1, // exclude sticky posts
                   'orderby' => $order,
                   'posts_per_page' => $number,
                  );
    $posts->query(apply_filters('atom_featured_posts_query', $query, $args, $this));

    $items = array();
    $count = 0;
    while ($posts->have_posts()):
      $posts->the_post();
      $items[$count] = $post;
      $count++;
    endwhile;
    if($max_columns > 0) $items = $this->array_chunk_fixed($items, $max_columns);

    // restore original query so we don't interfere with other loops
    wp_reset_query();
    remove_filter("posts_where", $where);

    return empty($posts) ? false : $items; // return the items array if posts were found, otherwise false
  }

  function widget($args, $instance){
    $cache = wp_cache_get('widget_featured_posts', 'widget');
    if (!is_array($cache)) $cache = array();
    if (isset($cache[$args['widget_id']])):
      echo $cache[$args['widget_id']];
      return;
    endif;

    extract($args, EXTR_SKIP);
    $instance = wp_parse_args($instance, $this->defaults());

    $posts = $this->get_featured_posts($instance);
    if(!$posts) return atom_add_debug_message("No relevant featured posts found {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");

    ob_start();
    echo $before_widget;
    ?>

    <!-- featured posts -->
    <div class="featured-posts">
     <div class="block-content clear-block">

       <?php if((count($posts) > 5) && $instance['navigation']): // enable navigation only if we have more than 5 posts ?>
       <a class="prev">&laquo;</a>
       <a class="next">&raquo;</a>
       <?php endif; ?>

       <ul class="items">
         <?php foreach ($posts as $column => $entries): ?>
          <li class="item <?php echo (count($posts) > 1 ? 'cycle' : '').(($column == count($posts)-1) ? ' last' : ''); ?>">
            <?php
              foreach ($entries as $entry)

                // output template
                echo atom_render_template($instance['template'], array(
                  'TITLE' => atom_get_post_title($entry->ID, 56),
                  'COMMENT_COUNT' => $entry->comment_count,
                  'URL' => atom_get_post_title_url($entry->ID),
                  'THUMBNAIL' => atom_get_post_thumb($entry->ID, 'featured-thumbnail'),
                  'CONTENT' => convert_smilies(atom_filter_content($entry->post_content, array(
                     'limit' => $instance['word_count'],
                     'allowed_tags' => explode(',', $instance['$allowed_tags']),
                     'more' => $instance['content_filter_more'],
                    ))),
                  'ID' => $entry->ID,
                 ));
            ?>
          </li>
         <?php endforeach; ?>
       </ul>
     </div>
    </div>
    <!-- /featured posts -->

    <?php
    echo $after_widget;
    $output = ob_get_clean();

    echo $output;
    $cache[$args['widget_id']] = $output;
    wp_cache_set('widget_featured_posts', $cache, 'widget');
  }

  function update($new_instance, $old_instance){
    $instance = $old_instance;
    $instance['source'] = esc_attr($new_instance['source']);
    $instance['category'] = intval($new_instance['category']);
    $instance['number'] = intval($new_instance['number']);
    $instance['number'] = min(max(intval($new_instance['number']), 1), 20);
    $instance['max_columns'] = min(max(intval($new_instance['max_columns']), 1), 8); // 8 should be more than enough
    $instance['navigation'] = (bool)$new_instance['navigation'];
    $instance['timeframe'] = intval($new_instance['timeframe']);
    $instance['delay'] = min(max(intval($new_instance['delay']), 1), 600);
    $instance['effect'] = esc_attr($new_instance['effect']);
    $instance['easing'] = esc_attr($new_instance['easing']);

    // html template
    if(current_user_can('edit_themes')) $instance['template'] = $new_instance['template'];

    $this->flush_cache();
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_featured_posts_defaults', array(
      'source' => 'marked',
      'category' => 0,
      'number' => 15,
      'navigation' => false,
      'timeframe' => 60,
      'delay' => 10,
      'effect' => 'scrollDown',
      'easing' => 'easeOutBack',
      'template' => $this->template(),
      'max_columns' => 5,

      // hidden
      'order' => 'rand',
      'word_count' => 10,
      'allowed_tags' => 'abbr,acronym,b,cite,code,del,dfn,em,i,ins,q,strong,sub,sup',
      'content_filter_more' => '[&hellip;]',
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>

    <?php if(!atom_get_options('jquery')): ?>
      <div <?php atom_widget_form_class($this); ?>><p class="widget-error"><?php _ae("This widget requires jQuery to be enabled"); ?></p></div>
      <?php return;
    endif;?>

    <div <?php atom_widget_form_class($this); ?>>

    <div class="high-priority-block">

      <p><strong><em><?php _ae("Content Source:"); ?></em></strong></p>
      <input type="radio" followRules value="marked" id="<?php echo $this->get_field_id('source'); ?>_marked" name="<?php echo $this->get_field_name('source'); ?>" <?php checked($instance['source'], 'marked'); ?> />
      <label for="<?php echo $this->get_field_id('source'); ?>_marked"><?php printf(_a("%s that I mark as featured"), '<a href="'.admin_url('edit.php').'">'._a("Posts").'</a>'); ?></label>
      <br />
      <input id="<?php echo $this->get_field_id('source'); ?>_category" name="<?php echo $this->get_field_name('source'); ?>" type="radio" followRules value="category" <?php checked($instance['source'], 'category'); ?> />
      <?php
       $fc = atom_dropdown_terms(array(
         'name' => $this->get_field_name('category'),
         'id' => $this->get_field_id('category'),
         'selected' => intval($instance['category']),
         'show_option_all' => _a('-- All categories --'),
         'hide_empty' => 0,
         'orderby' => 'name',
         'show_count' => 1,
         'echo' => 0,
         'hierarchical' => 1,
         'extra_attributes' => 'followRules rules="DEPENDS ON '.$this->get_field_name('source').' BEING category"',
       ));
      ?>
      <label for="<?php echo $this->get_field_id('source'); ?>_category"><?php printf(_a("Posts from category %s"), $fc); ?></label>
    </div>

    <p>
     <label for="<?php echo $this->get_field_id('number'); ?>"><?php _ae('How many entries to display?'); ?></label>
     <input id="<?php echo $this->get_field_id('number'); ?>" name="<?php echo $this->get_field_name('number'); ?>" type="text" value="<?php if (isset($instance['number'])) echo intval($instance['number']); ?>" size="3" />
    </p>

    <p>
     <?php $input = '<input id="'.$this->get_field_id('max_columns').'" name="'.$this->get_field_name('max_columns').'" type="text" value="'.(isset($instance['max_columns']) ? intval($instance['max_columns']) : null).'" size="3" />'; ?>
     <label for="<?php echo $this->get_field_id('max_columns'); ?>"><?php printf(_a('Limit to %s entries per slide'), $input); ?></label>
     <br />
     <small><?php _ae("Only change this value if you're using a fluid layout"); ?></small>
    </p>    

    <p>
     <?php $input = '<input id="'.$this->get_field_id('timeframe').'" name="'.$this->get_field_name('timeframe').'" type="text" value="'.(isset($instance['timeframe']) ? intval($instance['timeframe']) : null).'" size="3" />'; ?>
     <label for="<?php echo $this->get_field_id('timeframe'); ?>"><?php printf(_a('Exclude content older than %s day(s)'), $input); ?></label>
    </p>

    <p>
     <?php $input = '<input id="'.$this->get_field_id('delay').'" name="'.$this->get_field_name('delay').'" type="text" value="'.(isset($instance['delay']) ? intval($instance['delay']) : null).'" size="3" />'; ?>
     <label for="<?php echo $this->get_field_id('delay'); ?>"><?php printf(_a('Cycle every %s second(s)'), $input); ?></label>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('navigation'); ?>">
     <input type="checkbox" id="<?php echo $this->get_field_id('navigation'); ?>" name="<?php echo $this->get_field_name('navigation'); ?>"<?php checked($instance['navigation']); ?> />
 	   <?php _ae('Show Previous/Next Links'); ?></label>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('effect'); ?>"><?php _ae('Transition effect:'); ?></label>
      <select id="<?php echo $this->get_field_id('effect'); ?>" name="<?php echo $this->get_field_name('effect'); ?>">
       <?php foreach(unserialize(EFFECTS) as $fx => $name): ?>
       <option value="<?php echo $fx; ?>" <?php selected($instance['effect'], $fx); ?>><?php echo $name; ?></option>
       <?php endforeach; ?>
      </select>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('easing'); ?>"><?php _ae('Easing:'); ?></label>
      <select id="<?php echo $this->get_field_id('easing'); ?>" name="<?php echo $this->get_field_name('easing'); ?>">
       <?php foreach(unserialize(EASING) as $fx => $name): ?>
       <option value="<?php echo $fx; ?>" <?php selected($instance['easing'], $fx); ?>><?php echo $name; ?></option>
       <?php endforeach; ?>
      </select>
    </p>

    <?php if(current_user_can('edit_themes')): ?>
    <div class="high-priority-block">
      <p>
      <label for="<?php echo $this->get_field_id('template'); ?>"><strong><?php _ae('Entry Template (HTML):'); ?></strong>
      <textarea class="widefat code" id="<?php echo $this->get_field_id('template'); ?>" name="<?php echo $this->get_field_name('template'); ?>" rows="8" cols="28"><?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?></textarea>
      </label>
      </p>
    </div>
    <?php endif; ?>

    </div>
  <?php

  }
}