<?php

/**
 * Latest Comments
 * Differences to the default one:
 *  + Template system
 *  + Limit word count
 *  + Show additional content: avatars, date, country flag
 *  + Retrieve more comments via ajax
 *  + Site wide data (heavy)
 *
 * @since 1.0
 * @todo test cache
 * @todo test the comment content filter
 */
 
 
unregister_widget('WP_Widget_Recent_Comments');   
register_widget('atom_widget_recent_comments');

class atom_widget_recent_comments extends WP_Widget{

  function atom_widget_recent_comments(){
    $widget_ops = array('classname' => 'recent-comments', 'description' => _a('The most recent comments'));
    $control_ops = array('width' => 500);
    $this->WP_Widget('atom-recent-comments', _a('Recent Comments'), $widget_ops, $control_ops);

    atom_add('ajax_requests', array(&$this, 'ajax'));

    // run after the widget instance is properly registered (we only need to define the COUNTRY_FLAGS constant)
    add_action('widgets_init', array(&$this, 'set_country_flags'), 99);

    // include in jQuery(document).ready()
    atom_add('jquery_init', array(&$this, 'js'));

    // flush cache when comments are changed
    add_action('comment_post', array(&$this, 'flush_cache'));
    add_action('transition_comment_status', array(&$this, 'flush_cache'));
  }

  function set_country_flags(){
    // we need to process all instances because this function gets to run only once
    $widget_settings = atom_get_options($this->option_name);

    foreach((array)$widget_settings as $instance => $options):

      // identify instance
      $id = "{$this->id_base}-{$instance}";
      $block_id = "instance-{$id}";

      // we need to set a custom margin value to the list text, if the widget shows avatars
      if(!defined('COUNTRY_FLAGS') && isset($options['country_flag']) && $options['country_flag'] && is_active_widget(false, $id, $this->id_base)) define('COUNTRY_FLAGS', TRUE);

    endforeach;
  }

  function flush_cache(){
    wp_cache_delete('widget_recent_comments', 'widget');
  }

  function js(){
    // we need to process all instances because this function gets to run only once
    $widget_settings = atom_get_options($this->option_name);

    foreach((array)$widget_settings as $instance => $options):

      // identify instance
      $id = "{$this->id_base}-{$instance}";
      $block_id = "instance-{$id}";

      if (isset($options['more']) && $options['more'] && !$options['site_wide'] && is_active_widget(false, $id, $this->id_base)): ?>

      $("#<?php echo $block_id; ?> a.more").click(function(event){
        event.preventDefault();

        $offset = parseInt($(this).attr('rel'));

        $.ajax({
          type: "GET",
          url: "<?php echo home_url(); //echo atom_get_current_page_url(); ?>",
          data: { instance: <?php echo $instance; ?>,
                  offset: $offset,
                  _ajax_nonce: "<?php atom_nonce('get_recent_comments'); ?>",
                  atom: 'get_recent_comments' },
          dataType: 'json',
          context: this,
          beforeSend: function() { $(this).addClass("loading"); },
          complete: function() { $(this).removeClass("loading"); },
          success: function(response){

            var link = $(this);

            // append to list & update tab container height (if we're inside a tabbed widget)
            if(response.output != '')
              $(response.output).appendTo("#<?php echo $block_id; ?> ul").hide().each(function(i){
                 $(this).delay(333*i).animate(
                    {
                      "opacity": "show",
                      "height": "show",
                      "marginTop": "show",
                      "marginBottom": "show",
                      "paddingTop": "show",
                      "paddingBottom": "show"
                    },
                    { duration: 333,
                      step: function(now, fx){
                      link.parents('li.block .sections').height((link.parents('#<?php echo $block_id; ?>').height()) + 5);
                    }
                 });
              });

            link.attr('rel', response.offset);

            // no more data?
            if(!response.more) link.hide();

          }
        });
      });

      <?php endif;
    endforeach;
  }

  function ajax(){
    if(atom_ajax_request('get_recent_comments')):
      atom_ajax_header('get_recent_comments', 'application/json');

      $settings = atom_get_options($this->option_name);
      $instance = intval($_GET['instance']);
      $settings = $settings[$instance];

      $offset = isset($_GET['offset']) ? intval($_GET['offset']) : 0;
      $output = $this->get_comments($settings, $next, $offset);
      $offset = $offset + $settings['number'];
      //$next = ($this->get_comments($settings, $offset)) ? true : false;

      echo json_encode(array('output' => $output, 'more' => $next, 'offset' => $offset));
      exit();
    endif;
  }

  function template(){
    return "<a class=\"fadeThis clear-block\" href=\"{URL}\" title=\"".sprintf(_a('on %s'), '{TITLE}')."\">\n"
              ." {AVATAR}\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{FLAG}{AUTHOR}</span>\n"
              ."   <span class=\"c1\">{CONTENT}</span>\n"
              ."   <span class=\"c2\">{DATE}</span>\n"
              ." </span>\n"
              ."</a>\n";
  }

  function get_comments($args, &$more, $offset = 0){
    extract($args);

    if($site_wide):
      global $wpdb;
      $selects = array();

      foreach (atom_get_sites() as $blog)
        $selects[] = "(SELECT comment_ID, comment_post_ID, comment_author, comment_author_email, comment_author_url, comment_author_IP, comment_date_gmt, comment_content, post_title, {$blog['blog_id']} as blog_id FROM ".$wpdb->get_blog_prefix($blog['blog_id'])."comments
          LEFT JOIN ".$wpdb->get_blog_prefix($blog['blog_id'])."posts
          ON comment_post_id = id
          WHERE post_status = 'publish'
            AND post_password = ''
            AND comment_approved = '1'
            AND comment_type = ''
           ORDER BY comment_date_gmt DESC LIMIT {$number})"; // real number is (number * # of blogs)

      $comments = $wpdb->get_results(implode(" UNION ALL ", $selects)." ORDER BY comment_date_gmt DESC", OBJECT);
    else:
      $comments = get_comments(array('number' => $number+1, 'status' => 'approve', 'offset' => intval($offset)));
    endif;

    $more = count($comments) > $number ? true : false;
    $count = 1;
    $output = '';
    foreach((array)$comments as $comment):
      if($count++ == $number+1) break;
      $output .= '<li'.($count == 1 && $offset == 0  ? ' class="first"' : NULL).'>';

      // output template
      $output .= atom_render_template($args['template'], array(
        'TITLE' => $site_wide ? strip_tags($comment->post_title) : get_the_title($comment->comment_post_ID),
        'URL' => $site_wide ? get_blog_permalink($comment->blog_id, $comment->comment_post_ID) : esc_url(get_comment_link($comment->comment_ID)), // @todo: get comment page for MU urls
        'AVATAR' => atom_get_avatar($comment->comment_author_email, $avatar_size, '', $comment->comment_author),
        'FLAG' => (($country_flag && $country = atom_get_country($comment->comment_author_IP)) ? "<abbr title=\"{$country['name']}\" class=\"flag {$country['code']}\">&nbsp;</abbr>" : null),
        'AUTHOR' => $comment->comment_author,
        'CONTENT' => convert_smilies(atom_filter_content($comment->comment_content, array(
            'limit' => $word_count,
            'allowed_tags' => explode(',', $allowed_tags),
            'more' => $content_filter_more
          ))),
        'DATE' => atom_time_since(abs(strtotime($comment->comment_date_gmt))),
        'EMAIL' => esc_url($comment->comment_author_email), // should not be used
        'ID' => $comment->comment_ID,
       ));

      $output .= '</li>';
    endforeach;

    return $output;
  }

  function widget($args, $instance){
    $cache = wp_cache_get('widget_recent_comments', 'widget');
    if (!is_array($cache)) $cache = array();
    if (isset($cache[$args['widget_id']])):
      echo $cache[$args['widget_id']];
      return;
    endif;

    extract($args, EXTR_SKIP);
    $instance = wp_parse_args($instance, $this->defaults());

    $comments = $this->get_comments($instance, $next);
    if(!$comments) return atom_add_debug_message("No comments were found in {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");;;

    $title = apply_filters('widget_title', $instance['title'], $instance, $this->id_base);

    $output = $before_widget;
    if ($title) $output .= $before_title.$title.$after_title;

    $output .= "<ul class=\"menu full recent-comments\">{$comments}</ul>";
    if($instance['more'] && !$instance['site_wide'] && $next && atom_get_options('jquery'))
      $output .= "<a href=\"#\" class=\"fadeThis more\" rel=\"{$instance['number']}\">"._a("Show More")."</a>";

    $output .= $after_widget;

    echo $output;
    $cache[$args['widget_id']] = $output;
    wp_cache_set('widget_recent_comments', $cache, 'widget');
  }

  function update($new_instance, $old_instance){
    $instance = $old_instance;
    $instance['title'] = esc_attr($new_instance['title']);
    $instance['number'] = min(max(intval($new_instance['number']), 1), 20);                                         
    $instance['word_count'] = intval($new_instance['word_count']);
    $instance['avatar_size'] = intval($new_instance['avatar_size']);
    $instance['country_flag'] = (bool)$new_instance['country_flag'];
    $instance['more'] = (bool)$new_instance['more'];

    // html template
    if(current_user_can('edit_themes')) $instance['template'] = $new_instance['template'];

    // mu option
    if(atom_mu_network()) $instance['site_wide'] = (bool)$new_instance['site_wide'];

    $this->flush_cache();
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_recent_comments_defaults', array(
      'title' => _a("Recent Comments"),
      'number' => 5,
      'word_count' => 20,
      'avatar_size' => 48,
      'country_flag' => false,
      'more' => true,
      'template' => $this->template(),
      'site_wide' => false,

      // internal settings
      'allowed_tags' => 'abbr,acronym,b,cite,code,del,dfn,em,i,ins,q,strong,sub,sup',
      'content_filter_more' => '[&hellip;]',
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>
    <div <?php atom_widget_form_class($this); ?>>

    <?php if(atom_mu_network()): // only on mu ?>
    <div class="high-priority-block">
    <p>
     <input type="checkbox" id="<?php echo $this->get_field_id('site_wide'); ?>" name="<?php echo $this->get_field_name('site_wide'); ?>" <?php checked($instance['site_wide']); ?> followRules />
     <label for="<?php echo $this->get_field_id('site_wide'); ?>"><strong><?php _ae('Network (site-wide) content'); ?></strong></label>
     <br />
     <small><?php _ae("(heavy database usage, use caching to improve performance)"); ?></small>
    </p>
    </div>
    <?php endif; ?>

    <p>
     <label for="<?php echo $this->get_field_id('title'); ?>"><?php _ae('Title:'); ?></label>
     <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php if (isset($instance['title'])) echo esc_attr($instance['title']); ?>" />
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('number'); ?>"><?php _ae('How many entries to display?'); ?></label>
     <input id="<?php echo $this->get_field_id('number'); ?>" name="<?php echo $this->get_field_name('number'); ?>" type="text" value="<?php if (isset($instance['number'])) echo intval($instance['number']); ?>" size="3" />
    </p>

    <p>
     <?php $input = '<input id="'.$this->get_field_id('word_count').'" name="'.$this->get_field_name('word_count').'" type="text" value="'.(isset($instance['word_count']) ? intval($instance['word_count']) : null).'" size="3" />'; ?>
     <label for="<?php echo $this->get_field_id('word_count'); ?>"><?php printf(_a('Content has max %s word(s)'), $input); ?></label>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('avatar_size'); ?>"><?php _ae('Avatar Size:') ?></label>
      <input type="text" size="3" id="<?php echo $this->get_field_id('avatar_size'); ?>" name="<?php echo $this->get_field_name('avatar_size'); ?>" value="<?php if (isset($instance['avatar_size'])) echo intval($instance['avatar_size']); ?>" /> <?php _ae("pixels"); ?>
    </p>

    <p>
     <input type="checkbox" id="<?php echo $this->get_field_id('country_flag'); ?>" name="<?php echo $this->get_field_name('country_flag'); ?>"<?php checked($instance['country_flag']); ?> />
     <label for="<?php echo $this->get_field_id('country_flag'); ?>"><?php _ae('Enable Country Flags'); ?></label>
    </p>

    <p>
     <input <?php if(!atom_get_options('jquery')) echo "disabled=\"disabled\""; ?> type="checkbox" id="<?php echo $this->get_field_id('more'); ?>" name="<?php echo $this->get_field_name('more'); ?>"<?php checked($instance['more']); ?> followRules rules="CONFLICTS WITH <?php echo $this->get_field_name('site_wide'); ?>" />
     <label for="<?php echo $this->get_field_id('more'); ?>" <?php if(!atom_get_options('jquery')) echo "class=\"disabled\""; ?>><?php printf(_a('Display %s Link'), '<code>'._a("Show More").'</code>'); ?></label>
    </p>

    <?php if(current_user_can('edit_themes')): ?>
    <div class="high-priority-block">
      <p>
      <label for="<?php echo $this->get_field_id('template'); ?>"><strong><?php _ae('Entry Template (HTML):'); ?></strong></label>
      <textarea class="widefat code" id="<?php echo $this->get_field_id('template'); ?>" name="<?php echo $this->get_field_name('template'); ?>" rows="8" cols="28"><?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?></textarea>
      </p>
    </div>
    <?php endif; ?>
    </div>
  <?php
  }
}