<?php
/**
 * Tabs.
 * Display other widget instances (from the arbitrary area) as tabbed content inside this widget.
 *
 * @link http://digitalnature.ro
 *
 * @package ATOM
 * @subpackage Template
 *
 *
 * @since 1.0
 *
 * @todo add the possibility to drag & drop widgets inside this widget
 */



register_widget('atom_widget_tabs'); 
 
class atom_widget_tabs extends WP_Widget {
  function atom_widget_tabs() {
   $widget_ops = array('classname' => 'tabs', 'description' => _a("Group arbitrary widgets into tabs"));
   $this->WP_Widget('atom-tabs', _a('Tabbed Widgets'), $widget_ops);
  }

  function widget($args, $instance){
    if(!atom_get_options('jquery')) return atom_add_debug_message("jQuery required by {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");

    extract($args);
    $instance = wp_parse_args($instance, $this->defaults());

    $active_tabs = is_array($instance['widgets']) ? $instance['widgets'] : array();
    $all_widgets = $this->get_arbitrary_widgets();
    $cookie = isset($_COOKIE["tabs-$this->id"]) ? $_COOKIE["tabs-$this->id"] : false;

    // build fx class
    $instance['effect'] = ' '.str_replace(',', '', $instance['effect']);
    $instance['effect'] = str_replace(' ', ' fx-', $instance['effect']);

    $output = array();
    $count = 0;
    foreach($active_tabs as $w)
     foreach($all_widgets as $a)
       if(($a['id'] == $w) && ($widget = atom_get_widget($a['id']))):
         $count++;
         $classes = "nav-".implode(' ', $a['class']);
         if($count == 1) $classes .= " first"; elseif($count == count($active_tabs)) $classes .= " last";
         if(($cookie == "nav-{$a['id']}") || (!$cookie && $count == 1)) $classes .= " active";
         $title = apply_filters('atom_widget_tabs_title', ($a['title'] ? $a['title'] : '&nbsp;'), $a, $instance);
         $hidden = (($cookie == $classes) || (!$cookie && $count == 1)) ? '' : "hidden";
         $output[$count]['tab'] = "<li class=\"{$classes}\" id=\"nav-{$a['id']}\"><a href=\"#tab-{$a['id']}\">{$title}</a></li>\n";
         $output[$count]['content'] = "<div class=\"section {$hidden}\" id=\"tab-{$a['id']}\">\n{$widget}\n</div>\n";
       endif;

    if(empty($active_tabs) || empty($output)) return atom_add_debug_message("Tabbed widgets are not configured or missing content in {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");

    echo $before_widget;
    ?>
    <div class="widget-tabs <?php echo $instance['effect']; ?> ease-<?php echo $instance['easing']; ?>" id="tabs-<?php echo $this->id; ?>">
     <ul class="tabs clear-block">
      <?php foreach($output as $widget) echo $widget['tab']; ?>
     </ul>
     <div class="sections clear-block">
      <?php foreach($output as $widget) echo $widget['content']; ?>
     </div>
    </div>
    <?php
    echo $after_widget;
  }

  function update($new_instance, $old_instance){
    $instance['widgets'] = $new_instance['widgets'];
    $instance['effect'] = esc_attr($new_instance['effect']);
    $instance['easing'] = esc_attr($new_instance['easing']);
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_tabs_defaults', array(
      'widgets' => array(),
      'effect' => 'fade',
      'easing' => 'easeOutExpo',
    ), $this);
  }

  function form($instance){

    if(!atom_get_options('jquery')): ?>
      <div class="atom-block"><p class="widget-error"><?php _ae("This widget requires jQuery to be enabled"); ?></p></div>
      <?php return;
    endif;

    $instance = wp_parse_args($instance, $this->defaults());
    $active_tabs = is_array($instance['widgets']) ? $instance['widgets'] : array();
    ?>

    <div <?php atom_widget_form_class($this); ?>>
    <?php

    if(is_int($this->number)):
      $all_widgets = $this->get_arbitrary_widgets();

      if(empty($all_widgets)):
        echo '<p class="widget-error">'._a("To add tabs, you must have at least one active arbitrary widget. Click save to refresh...").'</p>';
      else:

        // sort
        $ordered_widgets = array();
        foreach($active_tabs as $w)
          foreach($all_widgets as $i => $a)
            if($w == $a['id']):
               $ordered_widgets[] = $a;
               unset($all_widgets[$i]);
             endif;
        $ordered_widgets = array_merge($ordered_widgets, $all_widgets);

        $sidebars_widgets = wp_get_sidebars_widgets();
        echo '<p><em>'._a("Create Tabs for:").'</em></p>';
        echo "<ul id=\"tabs_{$this->id}\" class=\"arbitrary-widget-list\">\n";

        foreach ($ordered_widgets as $w):
          // is the current widget checked?
          $checked = (in_array($w['id'], $active_tabs));

          // checkbox and label
          echo "<li class=\"entry ".($checked ? 'checked' : '')."\">\n<label>";
          echo "<input name=\"".$this->get_field_name('widgets')."[]\" type=\"checkbox\" value=\"{$w['id']}\" ".($checked ? 'checked="checked"' : '')." /> ";
          echo "<strong>{$w['name']}</strong>".($w['title'] ? ": {$w['title']}" : '');
          echo "</label>\n</li>\n";
        endforeach;

        echo "</ul>\n<br  />";

      endif; ?>
      <script type="text/javascript">
        //<![CDATA[

        jQuery(document).ready(function($){
          $("#tabs_<?php echo $this->id; ?>").sortable();
          $("#tabs_<?php echo $this->id; ?>").disableSelection();
        });

        //]]>
      </script>
      <?php
    endif;
    ?>

    <p>
      <label for="<?php echo $this->get_field_id('effect'); ?>"><?php _ae('Transition effect:') ?></label>
      <select class="widefat" id="<?php echo $this->get_field_id('effect'); ?>" name="<?php echo $this->get_field_name('effect'); ?>">
       <?php foreach(unserialize(EFFECTS) as $fx => $name): ?>
       <option value="<?php echo $fx; ?>" <?php selected($instance['effect'], $fx); ?>><?php echo $name; ?></option>
       <?php endforeach; ?>
      </select>
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('easing'); ?>"><?php _ae('Easing:') ?></label>
      <select class="widefat" id="<?php echo $this->get_field_id('easing'); ?>" name="<?php echo $this->get_field_name('easing'); ?>">
       <?php foreach(unserialize(EASING) as $fx => $name): ?>
       <option value="<?php echo $fx; ?>" <?php selected($instance['easing'], $fx); ?>><?php echo $name; ?></option>
       <?php endforeach; ?>
      </select>
    </p>
    </div>
    <?php
  }

  function get_arbitrary_widgets(){
    global $wp_registered_widgets;

    $sidebars_widgets = wp_get_sidebars_widgets();

    $matches = array();
    $count = 0;
    if(!empty($sidebars_widgets['arbitrary'])):
      foreach($sidebars_widgets['arbitrary'] as $index => $widget_id):

        // get the widget instance settings
        $options = atom_get_options($wp_registered_widgets[$widget_id]['callback'][0]->option_name);
        $options = $options[$wp_registered_widgets[$widget_id]['params'][0]['number']];

        // generate css classes, that can be used for styling the tabs with things like icons

        $id_base = $wp_registered_widgets[$widget_id]['callback'][0]->id_base;
        if($id_base == 'atom-splitter') continue; // no splitters here

        $classes = array();
        $classes[] = str_replace('atom-', '', $id_base); // first class (the widget id_base without "atom-")

        // add extra classes based on widget options for certain widgets
        // this is to allow different styling for widgets of the same type inside the tabs (for eg. recent/popular/random posts)
        switch($id_base):

          // order + post type if different than "post", + related if checked
          case 'posts':
            $classes[] = "posts-{$options['order_by']}";
            if($options['post_type'] != 'post') $classes[] = "posts-{$options['post_type']}";
            if($options['related']) $classes[] = "posts-related";
          break;

          // menu id
          case 'menu':
            $classes[] = "posts-{$options['nav_menu']}";
          break;

          // term taxonomy
          case 'tag_cloud':
            $classes[] = "tag_cloud-{$options['taxonomy']}";
          break;

          // term taxonomy
          case 'terms':
            $classes[] = "terms-{$options['taxonomy']}";
          break;

          // link category ID
          case 'links':
            if($options['category']) $classes[] = "links-{$options['category']}";
          break;

          // archives
          case 'archives':
            $classes[] = "archives-{$options['type']}";
          break;

          // blogs, sort order
          case 'blogs':
            $classes[] = "blogs-{$options['order_by']}";
          break;

        endswitch;

        $matches[$count]['id'] = $widget_id;
        $matches[$count]['number'] = $count;
        $matches[$count]['name'] = $wp_registered_widgets[$widget_id]['name'];
        $matches[$count]['title'] = isset($options['title']) ? $options['title'] : $matches[$count]['name'];
        $matches[$count]['class'] = apply_filters('atom_widget_tabs_classes', $classes, $options, $count);

        // use first class as slug (needed by the cookie to identify active tab class)
        $matches[$count]['slug'] = $id_base;

        $count++;
      endforeach;
      return $matches;
    endif;
    return array();
  }

}