<?php

/**
 * Display the categories
 * Differences to the default one:
 *  + Context-based category listing
 *  + Taxonomy selection
 *  + Order attribute
 *  + Ability to hide terms not associated with any posts
 *  + Ability exclude terms by ID
 *  + Depth selection
 *
 * @since 1.0
 *
 * @todo: test custom taxanomy dropdown
 */

unregister_widget('WP_Widget_Categories');  
register_widget('atom_widget_terms'); 

class atom_widget_terms extends WP_Widget{

  function atom_widget_terms(){
    $widget_ops = array('classname' => 'terms', 'description' => _a("A list of taxonomy terms like categories or post tags"));
    $control_ops = array('width' => 500);
    $this->WP_Widget('atom-terms', _a('Terms'), $widget_ops, $control_ops);
  }

  /* gets all term ancestors (if taxonomy is hierarchical) */
  /* part of the code snatched from: http://core.trac.wordpress.org/attachment/ticket/12443/get_ancestors.2.12443.diff */
  /* the complete function should be available in 3.1 */
  function get_ancestors($id = 0, $taxonomy = 'category') {
    $ancestors = array();
    if(is_taxonomy_hierarchical($taxonomy)):
      $term = get_term($id, $taxonomy);
      while (!is_wp_error($term) && !empty($term->parent) && !in_array($term->parent, $ancestors)):
        $ancestors[] = (int) $term->parent;
        $term = get_term($term->parent, $taxonomy);
      endwhile;
    endif;

    return $ancestors;
  }

  function template(){
    return "<a class=\"fadeThis clear-block\" href=\"{URL}\" title=\"".sprintf(_a('View all posts filed under %s'), '{NAME}')."\">\n"
              ." <span class=\"tt\">{NAME} <strong>({POST_COUNT})</strong></span>\n"
              ." <span class=\"c1\">{DESCRIPTION}</span>\n"
              ."</a>\n";
  }

  function term_entry($category, $args, $depth){
    $description = convert_smilies(atom_filter_content(apply_filters('category_description', $category->description, $category), array(
      'limit' => intval($args['description_word_limit']),
      'allowed_tags' => explode(',', $args['allowed_tags']),
      'more' => $args['content_filter_more']
    )));

    return atom_render_template($args['template'], array(
      'NAME' => apply_filters('list_cats', esc_attr($category->name), $category),
      'URL' => get_term_link($category, $category->taxonomy),
      'POST_COUNT' => intval($category->count),
      'DESCRIPTION' => $description,
      'ID' => $category->term_id,
      'RSS' => !empty($feed) ? get_term_feed_link($category->term_id, $category->taxonomy, $feed_type) : NULL
    ));
  }

  function widget($args, $instance){
    global $wp_query;

    extract($args);
    $instance = wp_parse_args($instance, $this->defaults());

    $title = apply_filters('widget_title', $instance['title'], $instance, $this->id_base);

    if($instance['type'] == 'sub' && is_taxonomy_hierarchical($instance['taxonomy'])):
      // only show on pages where the current category is set

      if (((($instance['taxonomy'] == 'category') && is_category()) || is_tax($instance['taxonomy'] || is_single())) === false)
        return atom_add_debug_message("Current category doesn't have children categories requested by {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");

      // $root = get_query_var('cat');
      $category = $wp_query->get_queried_object();
      $ancestor = end($this->get_ancestors($category->term_id, $instance['taxonomy']));

      if($instance['root'] && $ancestor) $root = $ancestor;  else $root = $category->term_id;
      $term = get_term($root, $instance['taxonomy']);

      $title = '<a href="'.get_term_link($term, $instance['taxonomy']).'">'.$term->name.'</a>';
    endif;

    echo $before_widget;
    if($title) echo $before_title.$title.$after_title;

    $query_args = array(
      'child_of' => isset($root) ? $root : 0,
      'orderby' => $instance['order_by'],
      'hierarchical' => $instance['hierarchical'] ? '1' : '0',
      'taxonomy' => $instance['taxonomy'],
      'hide_empty' => $instance['hide_empty'] ? '1' : '0',
      'title_li' => '',
      'exclude' => $instance['exclude'],
      'depth' => $instance['depth'],
      'walker' => $instance['dropdown'] ? '' : new atom_walker_terms(),
      'link-callback' => array(&$this, 'term_entry'),
      'template' => $instance['template'],
      'description_word_limit' => $instance['description_word_limit'],
      'allowed_tags' => $instance['allowed_tags'],
      'content_filter_more' => $instance['content_filter_more'],
    );
    if($instance['order_by'] == 'count') $query_args['order'] = 'DESC';

    $tax = get_taxonomy($instance['taxonomy']);
    if ($instance['dropdown'] ? '1' : '0'):
      $query_args['show_option_none'] = $tax->label.':';
      $query_args['class'] = 'wide';
      $query_args['name'] = $this->id.'_terms';
      $query_args['id'] = $this->id.'_terms';
      wp_dropdown_categories(apply_filters('widget_categories_dropdown_args', $query_args));
    ?>

    <script type='text/javascript'>
      /* <![CDATA[ */
      var dropdown = document.getElementById("<?php echo $this->id; ?>_terms");
      function onCatChange(){
       if (dropdown.options[dropdown.selectedIndex].value > 0)
        location.href = "<?php echo home_url(); ?>/?<?php echo ($instance['taxonomy'] == 'category' ? 'cat' : 'term_id'); ?>="+dropdown.options[dropdown.selectedIndex].value;
      }
      dropdown.onchange = onCatChange;
      /* ]]> */
    </script>

    <?php else: ?>
    <ul class="box menu">
     <?php wp_list_categories(apply_filters('widget_categories_args', $query_args)); ?>
    </ul>
    <?php
    endif;

    echo $after_widget;
  }

  function update($new_instance, $old_instance){
    $instance = $old_instance;
    $instance['title'] = esc_attr($new_instance['title']);
    $instance['taxonomy'] = esc_attr($new_instance['taxonomy']);
    $instance['type'] = esc_attr($new_instance['type']);
    $instance['root'] = (bool)$new_instance['root'];
    $instance['hierarchical'] = (bool)$new_instance['hierarchical'];
    $instance['dropdown'] = (bool)$new_instance['dropdown'];
    $instance['hide_empty'] = (bool)$new_instance['hide_empty'];
    $instance['order_by'] = esc_attr($new_instance['order_by']);
    $instance['depth'] = intval($new_instance['depth']);
    $instance['exclude'] = esc_attr($new_instance['exclude']);

    // html template
    if(current_user_can('edit_themes')) $instance['template'] = $new_instance['template'];

    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_terms_defaults', array(
       'title' => _a("Categories"),
       'taxonomy' => 'category',
       'type' => 'all',
       'root' => true,
       'dropdown' => 0,
       'hierarchical' => 1,
       'hide_empty' => 1,
       'order_by' => 'name',
       'depth' => 0,
       'exclude' => '',
       'template' => $this->template(),

       // internal settings
       'description_word_limit' => 10,
       'allowed_tags' => 'abbr,acronym,b,cite,code,del,dfn,em,i,ins,q,strong,sub,sup',
       'content_filter_more' => '[&hellip;]',
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>
    <div <?php atom_widget_form_class($this); ?>>
    <div class="high-priority-block">
      <p><strong><em><?php _ae("Display:"); ?></em></strong></p>
      <label for="<?php echo $this->get_field_id('type'); ?>_all">
        <input id="<?php echo $this->get_field_id('type'); ?>_all" name="<?php echo $this->get_field_name('type'); ?>" followRules value="all" type="radio" <?php checked($instance['type'], 'all'); ?> />
        <?php _ae('All Terms'); ?>
      </label>
      <br />
      <label for="<?php echo $this->get_field_id('type'); ?>_sub">
        <input id="<?php echo $this->get_field_id('type'); ?>_sub" name="<?php echo $this->get_field_name('type'); ?>" followRules value="sub" type="radio" <?php checked($instance['type'], 'sub'); ?> />
        <?php _ae('Children of the active term'); ?>
      </label>
      <br />
      <input style="margin-left: 20px;" id="<?php echo $this->get_field_id('root'); ?>" name="<?php echo $this->get_field_name('root'); ?>" type="checkbox" <?php checked(isset($instance['root']) ? $instance['root'] : 0); ?> followRules rules="DEPENDS ON <?php echo $this->get_field_name('type'); ?> BEING sub" />
      <label for="<?php echo $this->get_field_id('root'); ?>"><?php _ae('Start from root'); ?></label>

    </div>

    <p>
     <label for="<?php echo $this->get_field_id('title'); ?>"><?php _ae('Title:'); ?></label>
     <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($instance['title']); ?>" />
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('taxonomy'); ?>"><?php _ae('Taxonomy:'); ?></label>
     <select class="widefat" id="<?php echo $this->get_field_id('taxonomy'); ?>" name="<?php echo $this->get_field_name('taxonomy'); ?>">
      <?php foreach (get_object_taxonomies('post') as $taxonomy):
        $tax = get_taxonomy($taxonomy);
        if (empty($tax->labels->name)) continue; ?>
      <option value="<?php echo esc_attr($taxonomy) ?>" <?php selected($taxonomy, esc_attr($instance['taxonomy'])) ?>><?php echo $tax->labels->name; ?></option>
      <?php endforeach; ?>
     </select>
    </p>

    <p>
     <input type="checkbox" id="<?php echo $this->get_field_id('dropdown'); ?>" name="<?php echo $this->get_field_name('dropdown'); ?>" <?php checked(isset($instance['dropdown']) ? (bool)$instance['dropdown'] : false); ?> />
     <label for="<?php echo $this->get_field_id('dropdown'); ?>"><?php _ae('Show as dropdown'); ?></label>
     <br />
     <input type="checkbox" id="<?php echo $this->get_field_id('hierarchical'); ?>" name="<?php echo $this->get_field_name('hierarchical'); ?>" <?php checked(isset($instance['hierarchical']) ? (bool)$instance['hierarchical'] : false); ?> />
     <label for="<?php echo $this->get_field_id('hierarchical'); ?>"><?php _ae('Hierarchical?'); ?></label>
     <br />
     <input type="checkbox" id="<?php echo $this->get_field_id('hide_empty'); ?>" name="<?php echo $this->get_field_name('hide_empty'); ?>" <?php checked(isset($instance['hide_empty']) ? (bool)$instance['hide_empty'] : false); ?> />
     <label for="<?php echo $this->get_field_id('hide_empty'); ?>"><?php _ae('Hide empty?'); ?></label>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('order_by'); ?>"><?php _ae('Order by:'); ?></label>
     <select id="<?php echo $this->get_field_id('order_by'); ?>" name="<?php echo $this->get_field_name('order_by'); ?>">
      <option value="name" <?php selected('name', esc_attr($instance['order_by'])) ?>><?php _ae("Name"); ?></option>
      <option value="count" <?php selected('count', esc_attr($instance['order_by'])) ?>><?php _ae("Post count"); ?></option>
      <option value="ID" <?php selected('ID', esc_attr($instance['order_by'])) ?>><?php _ae("ID"); ?></option>
      <option value="slug" <?php selected('slug', esc_attr($instance['order_by'])) ?>><?php _ae("Slug"); ?></option>
     </select>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('exclude'); ?>"><?php _ae('Exclude (IDs separated by comma):'); ?></label>
     <input class="widefat" id="<?php echo $this->get_field_id('exclude'); ?>" name="<?php echo $this->get_field_name('exclude'); ?>" type="text" value="<?php echo esc_attr($instance['exclude']); ?>" />
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('depth'); ?>"><?php _ae('Depth:'); ?></label> <input type="text" value="<?php echo intval($instance['depth']); ?>" name="<?php echo $this->get_field_name('depth'); ?>" id="<?php echo $this->get_field_id('depth'); ?>" class="widefat" />
      <br />
      <small><?php _ae('0 = All levels'); ?></small>
    </p>

    <?php if(current_user_can('edit_themes')): ?>
    <div class="high-priority-block">
      <p>
      <label for="<?php echo $this->get_field_id('template'); ?>"><strong><?php _ae('Entry Template (HTML):'); ?></strong>
      <textarea class="widefat code" id="<?php echo $this->get_field_id('template'); ?>" name="<?php echo $this->get_field_name('template'); ?>" rows="8" cols="28"><?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?></textarea>
      </label>
      </p>
    </div>
    <?php endif; ?>
    </div>
    <?php
  }

}