<?php

/**
 * Alternative Text Widget
 * Differences to the default one:
 *  + Editing in visual mode (TinyMCE)
 *  + Run PHP code
 *
 * @since 1.0
 */


unregister_widget('WP_Widget_Text');
register_widget('atom_widget_text'); 
 
class atom_widget_text extends WP_Widget{

  function atom_widget_text() {
    $widget_ops = array('classname' => 'text', 'description' => (current_user_can('edit_themes') ? _a('Arbitrary text, HTML/PHP or shortcodes') : _a('Arbitrary text, HTML or shortcodes')));
    $control_ops = array('width' => 500);
    $this->WP_Widget('atom-text', _a('Text'), $widget_ops, $control_ops);

    add_action('admin_print_scripts-widgets.php', array(&$this, 'js'));
  }

  function mce_init($tinyMCEEditorID, $elementID){
		// add the custom init array
		$initSettingsArray = array (
		 	'mode' => 'exact',
			'elements' => $elementID,
			'cleanup_on_startup' => true,
			'language' => 'en',
			'theme' => 'advanced',
            'skin' => "wp_theme",
			'content_css' => THEME_URL.'/editor-style.css',
            'theme_advanced_buttons1' => "bold,italic,strikethrough,|,bullist,numlist,blockquote,hr,|,justifyleft,justifycenter,justifyright,|,link,unlink,image",
            'theme_advanced_buttons2' => "formatselect,underline,justifyfull,forecolor,backcolor,|,removeformat,|,charmap,|,outdent,indent,|,undo,redo",
			'theme_advanced_buttons3' => '',
			'theme_advanced_toolbar_location' => 'top',
			'theme_advanced_toolbar_align' => 'left',
			'theme_advanced_path' => true,
			'theme_advanced_statusbar_location' => 'bottom',

			'fix_list_elements' => true,
			'verify_css_classes' => true,
			'convert_fonts_to_spans' => true,
			'inline_styles' => true,
			'relative_urls' => false,
			'remove_script_host' => false,
			'entity_encoding' => 'raw',
			'add_form_submit_trigger' => true,

			'height' =>"280px",
			'width' =>"500px"
		);

		add_filter("tiny_mce_before_init", $initSettingsArray);

		$mce_options = '';
		foreach ($initSettingsArray as $k => $v) $mce_options .= $k.':"'.$v.'", ';
		$mce_options = rtrim(trim($mce_options), '\n\r,');

		return $mce_options;
  }

  function js(){
    wp_enqueue_script('tinymce', site_url()."/wp-includes/js/tinymce/tiny_mce.js", array('common', 'admin-widgets', 'jquery','wp-ajax-response', 'jquery-color'));
    wp_enqueue_script('atom-text-tinymce-admin-langs', site_url()."/wp-includes/js/tinymce/langs/wp-langs-en.js", array('tinymce'));
  }

  function widget($args, $instance){
    extract($args);
    $instance = wp_parse_args($instance, $this->defaults());
    $title = apply_filters('widget_title', $instance['title'], $instance, $this->id_base);
    $text = apply_filters('widget_text', $instance['text'], $instance);
    echo $before_widget;
    if (!empty($title)) echo $before_title.$title.$after_title;

    // evaluate php code if the user can edit themes
    if ($instance['php']):
      ob_start();
      eval("?>{$text}");
      $text = ob_get_clean();
    endif;
    ?>

    <div class="box textwidget"><?php echo $instance['filter'] ? wpautop($text) : $text; ?></div>
    <?php
    echo $after_widget;
  }

  function update($new_instance, $old_instance){
    $instance = $old_instance;
    $instance['title'] = esc_attr($new_instance['title']);
    $instance['visual'] = intval($new_instance['visual']);
    if (current_user_can('unfiltered_html')) $instance['text'] =  $new_instance['text'];
    else $instance['text'] = stripslashes(wp_filter_post_kses(addslashes($new_instance['text']))); // wp_filter_post_kses() expects slashed
    $instance['filter'] = isset($new_instance['filter']);
    if(current_user_can('edit_themes')) $instance['php'] = isset($new_instance['php']);
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    // default settings
    return apply_filters('atom_widget_text_defaults', array(
      'title' => '',
      'visual' => 0,
      'text' => '',
      'php' => true,
    ), $this);
  }

  function form($instance) {
    $instance = wp_parse_args($instance, $this->defaults());
    $title = esc_attr($instance['title']);
    $visual = intval($instance['visual']);
    $text = format_to_edit($instance['text']);
    $seed = rand(1, 999);

    ?>
    <div <?php atom_widget_form_class($this); ?>>
    <p>
     <label for="<?php echo $this->get_field_id('title'); ?>"><?php _ae('Title:'); ?></label>
     <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />

     <input type="hidden" class="visual-switch" id="<?php echo $this->get_field_id('visual'); ?>" name="<?php echo $this->get_field_name('visual'); ?>" value="<?php echo $visual; ?>" />
    </p>

    <div class="clear-block">

      <button class="alignright button-secondary toggle-editor <?php if(!is_numeric($this->number)): ?>needfix<?php endif; ?>"
         <?php if(is_numeric($this->number)): ?>onclick="toggleEditor_<?php echo $seed; ?>('<?php echo $this->get_field_id('text'); ?>');return false;" <?php endif; ?>>
      <?php _ae("Toggle Visual &lt;&gt; HTML"); ?>
      </button>
    </div>

    <div class="visual-editor">
      <textarea class="widefat code <?php if($visual): ?>hidden<?php endif; ?>" rows="16" cols="20" id="<?php echo $this->get_field_id('text'); ?>" name="<?php echo $this->get_field_name('text'); ?>"><?php echo $text; ?></textarea>
    </div>

    <p <?php if($visual == 1): ?>class="hidden"<?php endif; ?> id="text-options-<?php echo $seed; ?>">

     <label for="<?php echo $this->get_field_id('filter'); ?>">
       <input id="<?php echo $this->get_field_id('filter'); ?>" name="<?php echo $this->get_field_name('filter'); ?>" type="checkbox" <?php checked(isset($instance['filter']) ? $instance['filter'] : 0); ?> />
       <?php _ae('Automatically add paragraphs'); ?>
     </label>

     <?php if (current_user_can('edit_themes')): ?>
     <br />
     <label for="<?php echo $this->get_field_id('php'); ?>">
       <input id="<?php echo $this->get_field_id('php'); ?>" name="<?php echo $this->get_field_name('php'); ?>" type="checkbox" <?php checked(isset($instance['php']) ? $instance['php'] : 0); ?> />
       <?php printf(_a("Execute code within %s tags"),"<code>&lt;?php ?&gt;</code>"); ?>
     </label>
     <?php endif; ?>

    </p>

    <?php
    if(is_numeric($this->number)):
      $mce_options = $this->mce_init("tinyMCEPreInit_$seed", $this->get_field_id('text')); ?>
      <script type="text/javascript">
        //<![CDATA[
        function toggleEditor_<?php echo $seed; ?>(id){
          if (!tinyMCE.get(id)){
            jQuery("#<?php echo $this->get_field_id('text'); ?>, #text-options-<?php echo $seed; ?>").addClass('hidden');
            jQuery('#<?php echo $this->get_field_id('visual'); ?>').val('1');
            <?php if($visual == 0): ?>
            tinyMCE.init(tinyMCEPreInit_<?php echo $seed; ?>.mceInit);
            <?php else: ?>
            tinyMCE.execCommand('mceAddControl', false, id);
            <?php endif; ?>
          }else{
            jQuery("#<?php echo $this->get_field_id('text'); ?>, #text-options-<?php echo $seed; ?>").removeClass('hidden');
            jQuery('#<?php echo $this->get_field_id('visual'); ?>').val('0');
            tinyMCE.execCommand('mceRemoveControl', false, id);
          }
        }

        // initialise the editor
        tinyMCEPreInit_<?php echo $seed; ?> = {
          base : '<?php echo site_url(); ?>/wp-includes/js/tinymce',
          suffix : '',

          mceInit : {<?php echo $mce_options; ?>},
          load_ext : function(url,lang){var sl=tinymce.ScriptLoader;sl.markDone(url+'/langs/'+lang+'.js');sl.markDone(url+'/langs/'+lang+'_dlg.js');sl.markDone(url+'/themes/advanced/langs/'+lang+'.js');}
        };

        tinyMCEPreInit_<?php echo $seed; ?>.load_ext(tinyMCEPreInit_<?php echo $seed; ?>.base, 'en');

        <?php if($visual == 1): ?>
        tinyMCE.init(tinyMCEPreInit_<?php echo $seed; ?>.mceInit);
        <?php endif; ?>

        var save = jQuery('.widget-control-save', jQuery('#<?php echo $this->get_field_id('title'); ?>').parents());
        //save.unbind('click');
        save.click(function(){
          tinyMCE.triggerSave();
          //wpWidgets.save(jQuery(this).parents('.widget'), 0, 1, 0);
        });

        /* ]]> */
      </script> <?php
    endif; ?>

   </div> <?php
  }
}