<?php

/**
 * Top Commenters
 * slightly based on "Top Commenters Gravatar" plugin - http://suhanto.net/top-commenters-gravatar-widget-wordpress/
 *
 * @since 1.0
 * @todo add fade to image links in avatar-only mode
 */

register_widget('atom_widget_top_commenters');  
 
class atom_widget_top_commenters extends WP_Widget{
  function atom_widget_top_commenters(){
    $widget_ops = array('classname' => 'top-commenters', 'description' => _a("People who frequently comment the blog"));
    $control_ops = array('width' => 500);
    $this->WP_Widget('atom-top-commenters', _a('Top Commenters'), $widget_ops, $control_ops);

    // run after the widget instance is properly registered (we only need to define the COUNTRY_FLAGS constant)
    add_action('widgets_init', array(&$this, 'set_country_flags'), 99);

    // flush cache when comments are changed
    add_action('comment_post', array(&$this, 'flush_cache'));
    add_action('transition_comment_status', array(&$this, 'flush_cache'));
  }

  function set_country_flags(){
    // we need to process all instances because this function gets to run only once
    $widget_settings = atom_get_options($this->option_name);

    foreach((array)$widget_settings as $instance => $options):

      // identify instance
      $id = "{$this->id_base}-{$instance}";
      $block_id = "instance-{$id}";

      // we need to set a custom margin value to the list text, if the widget shows avatars
      if((isset($options['country_flag'])) && is_active_widget(false, $id, $this->id_base) && !defined('COUNTRY_FLAGS')) define('COUNTRY_FLAGS', TRUE);

    endforeach;
  }

  function flush_cache(){
    wp_cache_delete('widget_top_commenters', 'widget');
  }

  function template($mode = 'full'){
    switch($mode):
      case 'full':
        return "<a id=\"{ID}\" class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{NAME}\" rel=\"external nofollow\">\n"
              ." {AVATAR}\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{FLAG}{NAME}</span>\n"
              ."   <span class=\"c1\">{COMMENT_COUNT}</span>\n"
              ." </span>\n"
              ."</a>\n";
        break;
      case 'images':
        return "<a id=\"{ID}\" class=\"fadeThis clear-block tt\" href=\"{URL}\" title=\"{NAME} ({COMMENT_COUNT})\" rel=\"external nofollow\">\n"
              ." {AVATAR}\n"
              ."</a>\n";
        break;
      case 'brief':
        return "<a id=\"{ID}\" class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{NAME} ({COMMENT_COUNT})\" rel=\"external nofollow\">\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{NAME}</span>\n"
              ." </span>\n"
              ."</a>\n";
        break;
      case 'detailed':
        return "<a id=\"{ID}\" class=\"fadeThis clear-block\" href=\"{URL}\" title=\"{NAME}\" rel=\"external nofollow\">\n"
              ." <span class=\"base\">\n"
              ."   <span class=\"tt\">{NAME}</span>\n"
              ."   <span class=\"c1\">{COMMENT_COUNT}</span>\n"
              ." </span>\n"
              ."</a>\n";
        break;

    endswitch;
  }

  function widget($args, $instance){
    global $wpdb;

    $cache = wp_cache_get('widget_top_commenters', 'widget');
    if (!is_array($cache)) $cache = array();
    if (isset($cache[$args['widget_id']])):
      echo $cache[$args['widget_id']];
      return;
    endif;

    extract($args);
    $instance = wp_parse_args($instance, $this->defaults());
    $title = apply_filters('widget_title', $instance['title'], $instance, $this->id_base);
    $output = $before_widget;

    $sql = "SELECT comment_author, comment_author_email, comment_author_url, comment_author_IP, user_id, count(1) as counter from {$wpdb->comments} WHERE comment_approved = '1' AND comment_type != 'pingback'";

    // group by (only the first two are used):
    //  - comment_author (name, default)
    //  - comment_author_email (same users may have different emails)
    //  - comment_author_url (people can have multiple sites)
    //  - comment_author_IP (really useless, lots of people have dynamic IPs)
    $sql .= " GROUP BY comment_author, comment_author_email ORDER BY counter DESC, comment_date DESC LIMIT ".intval($instance['number']);
    $commenters = $wpdb->get_results($wpdb->prepare($sql));
    if(!$commenters) return atom_add_debug_message("No relevant entries found in {$args['widget_id']} ({$args['widget_name']}). Widget marked as inactive");;

    if($title) $output .= $before_title.$title.$after_title;

    $output .= "<ul class=\"menu top-commenters {$instance['mode']}\">";
    $count = 0;
    if($instance['mode'] == 'template') $template = $instance['template']; else $template = $this->template($instance['mode']);
    foreach($commenters as $commenter):
      $count++;
      $comment_count = isset($commenter->counter) ? sprintf(_an('%s comment', '%s comments', $commenter->counter), number_format_i18n($commenter->counter)) : '';
      //$author_has_url = !(empty($commenter->comment_author_url) || 'http://' == $commenter->comment_author_url);
      $output .= '<li'.($count == 1 ? ' class="first"' : NULL).'>';

      // output template
      $output .= atom_render_template($template, array(
        'ID' => 'a-'.uniqid(),
        'NAME' => $commenter->comment_author,
        'AVATAR' => atom_get_avatar($commenter->comment_author_email, intval($instance['avatar_size']), '', $commenter->comment_author),
        'URL' => esc_url($commenter->comment_author_url),
        'KARMA' => intval(get_user_meta($commenter->user_id, "karma", true)),
        'FLAG' => (($country = atom_get_country($commenter->comment_author_IP)) ? "<abbr title=\"{$country['name']}\" class=\"flag {$country['code']}\">&nbsp;</abbr>" : ''),
        'COMMENT_COUNT' => $comment_count,
        'EMAIL' => esc_url($commenter->comment_author_email) // should not be used
       ));

      $output .= '</li>';
    endforeach;
    $output .= '</ul>';

    $output .= $after_widget;

    echo $output;
    $cache[$args['widget_id']] = $output;
    wp_cache_set('widget_top_commenters', $cache, 'widget');
  }

  function update($new_instance, $old_instance){
    $instance['title'] = esc_attr($new_instance['title']);
    $instance['number'] = min(max(intval($new_instance['number']), 1), 50);
    $instance['mode'] = esc_attr($new_instance['mode']);
    $instance['avatar_size'] = intval($new_instance['avatar_size']);
    $instance['country_flag'] = (bool)$new_instance['country_flag'];

    // html template
    if(isset($new_instance['template']) && current_user_can('edit_themes')) $instance['template'] = $new_instance['template'];

    $this->flush_cache();
    return wp_parse_args($instance, $this->defaults());
  }

  function defaults(){
    return apply_filters('atom_widget_top_commenters_defaults', array(
      'title' => _a("Top Commenters"),
      'number' => 12,
      'mode' => 'avatars',
      'avatar_size' => 72,
      'country_flag' => false,
      'template' => $this->template(),
    ), $this);
  }

  function form($instance){
    $instance = wp_parse_args($instance, $this->defaults());
    ?>
    <div <?php atom_widget_form_class($this); ?>>
    <p>
      <label for="<?php echo $this->get_field_id('title'); ?>"><?php _ae('Title:') ?></label>
      <input type="text" class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" value="<?php if (isset($instance['title'])) echo esc_attr($instance['title']); ?>" />
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('number'); ?>"><?php _ae('How many entries to display?') ?></label>
      <input type="text" size="3" id="<?php echo $this->get_field_id('number'); ?>" name="<?php echo $this->get_field_name('number'); ?>" value="<?php if (isset($instance['number'])) echo intval($instance['number']); ?>" />
    </p>

    <p>
      <label for="<?php echo $this->get_field_id('avatar_size'); ?>"><?php _ae('Avatar Size:') ?></label>
      <input type="text" size="3" id="<?php echo $this->get_field_id('avatar_size'); ?>" name="<?php echo $this->get_field_name('avatar_size'); ?>" value="<?php if (isset($instance['avatar_size'])) echo intval($instance['avatar_size']); ?>" /> <?php _ae("pixels"); ?>
    </p>

    <p>
     <label for="<?php echo $this->get_field_id('country_flag'); ?>">
     <input type="checkbox" id="<?php echo $this->get_field_id('country_flag'); ?>" name="<?php echo $this->get_field_name('country_flag'); ?>"<?php checked($instance['country_flag']); ?> />
 	   <?php _ae('Enable Country Flags'); ?></label>
    </p>

    <div class="high-priority-block">
    <p>
      <label for="<?php echo $this->get_field_id('mode'); ?>"><strong><?php _ae('Display mode:') ?></strong></label><br />
      <select id="<?php echo $this->get_field_id('mode'); ?>" name="<?php echo $this->get_field_name('mode'); ?>" followRules>
       <option value="images" <?php selected($instance['mode'], "images"); ?>><?php _ae("Avatars only"); ?></option>
       <option value="brief" <?php selected($instance['mode'], "brief"); ?>><?php _ae("Text only"); ?></option>
       <option value="full" <?php selected($instance['mode'], "full"); ?>><?php _ae("Both Avatars and Text"); ?></option>
       <?php if(current_user_can('edit_themes')): ?>
       <option value="template" <?php selected($instance['mode'], "template"); ?>><?php _ae("Advanced, use template below"); ?></option>
       <?php endif; ?>
      </select>
    </p>
    </div>

    <?php if(current_user_can('edit_themes')): ?>
    <div class="high-priority-block">
      <p>
      <label for="<?php echo $this->get_field_id('template'); ?>"><strong><?php _ae('Entry Template (HTML):'); ?></strong>
      <input type="hidden" name="<?php echo $this->get_field_name('template'); ?>" value="<?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?>" />
      <textarea followRules rules="DEPENDS ON <?php echo $this->get_field_name('mode'); ?> BEING template" class="widefat code" id="<?php echo $this->get_field_id('template'); ?>" name="<?php echo $this->get_field_name('template'); ?>" rows="8" cols="28"><?php if (isset($instance['template'])) echo format_to_edit($instance['template']); ?></textarea>
      </label>
      </p>
    </div>
    <?php endif; ?>
    </div>
    <?php
  }

}